import java.util.Vector;
import java.util.Iterator;
import java.util.Map;

public interface Assertion  extends ProcessContainer
{
}

class BoolAssertion implements Assertion
{
	UnnamedProgram prog;
	String progName, condition;
	
	
	public BoolAssertion (Program p, BExpr b)
	{
		progName = p.name;
		condition= b.toString();
		
		
		prog = p.copy();
		
		//put the assertion in the top priority level. 
		//for efficiency we do not create a new priority level specially for it
		//(unless the list of processes is empty
		if (prog.procs.size() > 1) prog.procs.firstElement().add(new PriorityAssert( b ));
		else if (prog.procs.size() == 1) prog.procs.firstElement().add(new SimpleAssert(b));
		else throw new ParserException("BoolAssertion - Process List Empty");	
			
		//prog = new UnnamedProgram(procs);
	}
	
	
	//The Assert process must go in the highest priority level as we always want it to be able to make its check
	//If there are lower levels though it must generate noop events or it will stop them from running
	//otherwise we can just b once and since fdr explores every possible execution path it will pick up on 
	//any 1 path that fails the check.
	class PriorityAssert extends Iter
	{
		public PriorityAssert(BExpr b) {super(new Seq(new Atomic(new If(new Not(b), new Sig("assertionfailed"))),new NoOp()));}
		
		public String toString() {return "assertion";}
	}
	class SimpleAssert extends Atomic
	{
		public SimpleAssert(BExpr b) {super(new If(b, new Skip(), new Sig("assertionfailed")));}
		
		public String toString() {return "assertion";}
	}
	
	
	
	public String genCode(Context con)
	{
		return "assert CHAOS(diff(Events, {|assertionfailedT|})) [T= " + prog.genCode(con);
	}
	
	
	
	
	public int numIEs(Map<String,Integer> consts){return prog.numIEs(consts);}
	public int numBEs(Map<String,Integer> consts){return prog.numBEs(consts);}
	public int numProcs() {return prog.numProcs();}

	public String getIExpr(int i, Map<String, Integer> ivals) {return prog.getIExpr(i,ivals);}
	public String getBExpr(int i, Map<String, Integer> ivals,Map<String,Boolean> bvals) {return prog.getBExpr(i,ivals,bvals);}
	
	public String toString()
	{ return "assert always " + condition + " in " + progName;}


	
	public String process(int i) {return prog.process(i);}
	public String ownerOfIExpr(int i,Map<String,Integer> consts) { return prog.ownerOfIExpr(i,consts);}

	public String ownerOfBExpr(int i,Map<String,Integer> consts) {return prog.ownerOfBExpr(i,consts);}

	public int indexOfBExprOwner(int i,Map<String,Integer> consts){return prog.indexOfBExprOwner(i,consts);}
	
	public int indexOfIExprOwner(int i,Map<String,Integer> consts){return prog.indexOfIExprOwner(i,consts);}

}

class SignalAssertion implements Assertion
{
	Program prog;
	String signals;
	String renamedSignals;
	
	public SignalAssertion(Program prog, Vector<String> signals)
	{
		if (prog == null) throw new ParserException("prog argument to SignalAssertion null");
		if (signals == null) throw new ParserException("signals argument to SignalAssertion null");
		
		this.prog = prog;
		
		this.signals = "{";
		renamedSignals = "{|";
		Iterator<String> i = signals.iterator();
		while (i.hasNext())
		{
			String sig = i.next();
			this.signals = this.signals + sig;
			String renamedSig = sig.contains(".") ? sig.replaceFirst("\\.","T.") : sig + "T";
			
			
			renamedSignals = renamedSignals + renamedSig;
			
			if (i.hasNext())
				{this.signals = this.signals + ","; renamedSignals = renamedSignals + ",";}
		}
		this.signals = this.signals + "}";
		renamedSignals += "|}";
	}
	
	public String genCode(Context con)
	{
		return "assert CHAOS(diff(Events," + renamedSignals + ")) [T= " + prog.name;
	}

	
	
	public int numIEs(Map<String,Integer> consts){return prog.numIEs(consts);}
	public int numBEs(Map<String,Integer> consts){return prog.numBEs(consts);}
	public int numProcs() {return prog.numProcs();}


	public String getIExpr(int i, Map<String, Integer> ivals) {return prog.getIExpr(i,ivals);}
	public String getBExpr(int i, Map<String, Integer> ivals,Map<String,Boolean> bvals) {return prog.getBExpr(i,ivals,bvals);}
	
	
	public String toString()
	{
		return "assert nosignal " + signals + " in " + prog.name;
	}
	
	public String process(int i) {return prog.process(i);}
	public String ownerOfIExpr(int i, Map<String,Integer> consts) { return prog.ownerOfIExpr(i,consts);}

	public String ownerOfBExpr(int i, Map<String,Integer> consts) {return prog.ownerOfBExpr(i, consts);}
	public int indexOfBExprOwner(int i,Map<String,Integer> consts){return prog.indexOfBExprOwner(i,consts);}
	
	public int indexOfIExprOwner(int i,Map<String,Integer> consts){return prog.indexOfIExprOwner(i,consts);}

}
	
	
	
	
	
	
	
	
	
	
	
	
