import java.util.Map;

public interface Cmd extends CompilableProc
//to implement CompliableProc it really needs to contain no references to process parameters
{}

abstract class aCmd implements Cmd //checks for invalid arguments to getIExpr and getBExpr
{
	public String getIExpr(int i, Map<String, Integer> ivals) 
	{
		if (i <= 0) throw new ParserException("getIExpr(" + i + ") called for " + this + ". negative argument is invalid");
		else if (i > numIEs(ivals)) throw new ParserException("getIExpr(" + i + ") called for " + this + "containing only " + numIEs(ivals) + " IExprs");
		else
			return getIE(i, ivals);
	}
	public String getBExpr(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) 
	{
		if (i <= 0) throw new ParserException("getBExpr(" + i + ") called for " + this + ". negative argument is invalid");
		else if (i > numBEs(ivals)) throw new ParserException("getBExpr(" + i + ") called for " + this + "containing only " + numBEs(ivals) + " BExprs");
		else
			return getBE(i, ivals, bvals);
	}
	
	abstract protected String getIE(int i, Map<String, Integer> ivals);//these assume i is within the valid range
	abstract protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals);

	abstract public String toString();
}

class Skip extends aCmd
{
	public String genCode(Context con)
	{
		return "Skip";
	}

	//these should never be called
	protected String getIE(int i, Map<String, Integer> ivals){ throw new ParserException( "getIE(" + i + ") called for skip"); }
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals){ throw new ParserException( "getBE(" + i + ") called for skip"); }

	public String toString(){ return "skip";}


	public int numIEs(Map<String,Integer> consts){return 0;}
	public int numBEs(Map<String,Integer> consts){return 0;}


}

class NoOp extends aCmd
{
	public String genCode(Context con)
	{
		return "NoOp";
	}

	//these should never be called
	protected String getIE(int i, Map<String, Integer> ivals){ throw new ParserException( "getIE(" + i + ") called for NoOp"); }
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals){ throw new ParserException( "getBE(" + i + ") called for NoOp"); }

	public String toString(){ return "noop";}


	public int numIEs(Map<String,Integer> consts){return 0;}
	public int numBEs(Map<String,Integer> consts){return 0;}


}

class Seq extends aCmd
{
	Cmd c1, c2;
	public Seq(Cmd c1, Cmd c2) 
	{

		if (c1==null) throw new ParserException("c1 Argument to Seq null");
		if (c2==null) throw new ParserException("c2 Argument to Seq null");		
		this.c1 = c1; this.c2=c2;

	}

	public String genCode(Context con)
	{
		return "Sq.(" + c1.genCode(con) + "," + c2.genCode(con) + ")";
	}

	protected String getIE(int i, Map<String, Integer> ivals)
	{
		if (i <= c1.numIEs(ivals)) return c1.getIExpr(i,ivals);
		else return c2.getIExpr(i-c1.numIEs(ivals),ivals);
	}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals)
	{
		if (i <= c1.numBEs(ivals)) return c1.getBExpr(i,ivals,bvals);
		else return c2.getBExpr(i -c1.numBEs(ivals),ivals,bvals);
	}
	

	public int numIEs(Map<String,Integer> consts){return c1.numIEs(consts) + c2.numIEs(consts);}
	public int numBEs(Map<String,Integer> consts){return c1.numBEs(consts) + c2.numBEs(consts);}
	
	
	public String toString()
	{ return c1 + "; " + c2;}
	
}

class Iter extends aCmd
{
	Cmd c;
	public Iter(Cmd c) 
	{
		if (c==null) throw new ParserException("Argument to Iter null");	
		this.c = c;
	}
	
	public String genCode(Context con)
	{
		return "Iter." + c.genCode(con);
	}
	
	
	protected String getIE(int i, Map<String, Integer> ivals) {return c.getIExpr(i,ivals);}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) {return c.getBExpr(i,ivals,bvals);}
	public String toString() {return "iter { " + c + " }";} 
	
	public int numIEs(Map<String,Integer> consts){return c.numIEs(consts);}
	public int numBEs(Map<String,Integer> consts){return c.numBEs(consts);}
}

class While extends aCmd
{
	Cmd c;
	BExpr b;
	public While(BExpr b, Cmd c) 
	{
		if (b==null) throw new ParserException("b argument to While null");
		if (c==null) throw new ParserException("c argument to While null");
		this.b=b; this.c = c;
	}
	
	public String genCode(Context con)
	{
		return "While.(" + b.genCode(con) + "," + c.genCode(con) + ")";
	}
	
	
	public int numIEs(Map<String,Integer> consts) 
	{
		if ( b.isConstFalse(consts) ) //compiler will turn this into skip
			return 0;
		else return c.numIEs(consts);
	}
	public int numBEs(Map<String,Integer> consts)
	{	
		if ( b.isConstFalse(consts) )
			return 0;
		else return c.numBEs(consts) + ( (b.isConstTrue(consts) || b.isVar()) ? 0 : 1); 
	}
	
	protected String getIE(int i, Map<String, Integer> ivals) {return c.getIExpr(i,ivals);}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) { return (i <= c.numBEs(ivals)) ? c.getBExpr(i,ivals,bvals) : b.printWithVals(ivals,bvals);}
	public String toString() { return "while " + b + " do { " + c + " }";}
}


class DoWhile extends aCmd
{

	Cmd c;
	BExpr b;

	public DoWhile(Cmd c, BExpr b)
	{
		if (b==null) throw new ParserException("b argument to DoWhile null");
		if (c==null) throw new ParserException("c argument to DoWhile null");
		this.b=b; this.c = c;
	}
	
	public String genCode(Context con)
	{
		return "DoWhile.(" + c.genCode(con) + "," + b.genCode(con) + ")";
	}
	
	
	public int numIEs(Map<String,Integer> consts) 
	{
		return c.numIEs(consts);
	}
	public int numBEs(Map<String,Integer> consts)
	{	
		return c.numBEs(consts) + ( (b.isConstTrue(consts) || b.isVar()) ? 0 : 1); 
	}
	
	protected String getIE(int i, Map<String, Integer> ivals) {return c.getIExpr(i,ivals);}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) { return (i <= c.numBEs(ivals)) ? c.getBExpr(i,ivals,bvals) : b.printWithVals(ivals,bvals);}
	public String toString() { return "do { " + c + " } while " + b;}
}


class If extends aCmd
{
	Cmd c1, c2;
	BExpr b;
	public If(BExpr b, Cmd c1, Cmd c2) 
	{
		if (b==null) throw new ParserException("b argument to If null");
		if (c1==null) throw new ParserException("c1 argument to If null");
		if (c2==null) throw new ParserException("c2 argument to If null");
		
		this.b=b; this.c1 = c1; this.c2 = c2;

	}
	public If(BExpr b, Cmd c1) {this(b,c1,new Skip());}
	
	public String genCode(Context con)
	{
		return "Cond.(" + b.genCode(con) + "," + c1.genCode(con) + "," + c2.genCode(con) + ")";
	}
	
	protected String getIE(int i, Map<String, Integer> ivals)
	{
		if (b.isConstTrue(ivals))
			return c1.getIExpr(i, ivals);
		else if (b.isConstFalse(ivals))
			return c2.getIExpr(i, ivals);
		else if (i <= c1.numIEs(ivals)) 
			return c1.getIExpr(i, ivals);
		else /*if (i <= c1.numIEs(ivals) + c2.numIEs(ivals))*/
			return c2.getIExpr(i - c1.numIEs(ivals), ivals);

	}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals)
	{
		if (b.isConstTrue(ivals))
			return c1.getBExpr(i,ivals,bvals);
		else if (b.isConstFalse(ivals))
			return c2.getBExpr(i,ivals,bvals);
		else if (i <= c1.numBEs(ivals)) 
			return c1.getBExpr(i,ivals,bvals);
		else if (i <= c1.numBEs(ivals) + c2.numBEs(ivals))
			return c2.getBExpr(i - c1.numBEs(ivals),ivals,bvals);
		else
			return b.printWithVals(ivals,bvals);
	}
	
	public int numIEs(Map<String,Integer> consts) 
	{
		if (b.isConstTrue(consts))
			return c1.numIEs(consts);
		else if (b.isConstFalse(consts))
			return c2.numIEs(consts);
		else
			return c1.numIEs(consts) + c2.numIEs(consts);
	}
	public int numBEs(Map<String,Integer> consts)
	{
		if (b.isConstTrue(consts))
			return c1.numBEs(consts);
		else if (b.isConstFalse(consts))
			return c2.numBEs(consts);
		else
			return c1.numBEs(consts) + c2.numBEs(consts) + (b.isVar() ? 0 : 1);
	}
	
	public String toString() { return "if " + b + " then { " + c1 + " } else { " + c2 + " }"; }
}

class IAssign extends aCmd
{
	IVName ivname;
	IExpr iexpr;
	public IAssign(IVName ivname, IExpr iexpr) 
	{
		if (ivname==null) throw new ParserException("ivname argument to IAssign null");
		if (iexpr==null) throw new ParserException("iexpr argument to IAssign null");
		this.ivname = ivname; this.iexpr = iexpr;
	}
	
	public String genCode(Context con)
	{
		return "Iassign.(" + ivname.genCode(con) + "," + iexpr.genCode(con) + ")";
	}
	
	
	public int numIEs(Map<String,Integer> consts){return (iexpr.isConst() || iexpr.isVar()) ? 0 : 1;}
	public int numBEs(Map<String,Integer> consts){return 0;}
	
	protected String getIE(int i, Map<String, Integer> ivals) {return iexpr.printWithVals(ivals);}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) //this should never get called
	{
		throw new ParserException("IAssign.getBE(i) called but IAssigns do not contain BEs");
	}
	
	public String toString() { return ivname + ":= " + iexpr;}
}	

//Seperate names of variables used for assignment from variables appearing in expressions because:
//i) variables in expressions get tagged with IVar/BVar
//ii)integer "variables" in expressions may actually be constants passed to the parameter 
//NB ii) is no longer true
interface VName extends Genable {}

interface IVName extends VName {}

class IVarName implements IVName
{
	String name;
	public IVarName(String name)
	{
		if (name==null) throw new ParserException("name argument to IVarName null"); 
		this.name = name;
	}
	
	public String genCode(Context con) 
	{ 
		return con.getIEncoding(name);
	}
	
	public String toString() { return name;}

}

class IArrayName implements IVName
{
	String name;
	IConst i;
	public IArrayName(String name, IConst i)
	{
		if (name==null) throw new ParserException("name argument to IArrayName null"); 
		if (i==null) throw new ParserException("index argument to IArrayName null");
		
		this.name = name; this.i = i;
	}

	public String genCode(Context con) 
	{ 
		return con.getIAEncoding(name) + "." + i.genCode(con);
	}
	
	public String toString() { return name + "[" + i + "]";}
}

	
class BAssign extends aCmd
{
	BVName bvname;
	BExpr bexpr;
	public BAssign(BVName bvname, BExpr bexpr) 
	{
		if (bvname==null) throw new ParserException("bvname argument to BAssign null");
		if (bexpr==null) throw new ParserException("bexpr argument to BAssign null");
		 
		this.bvname = bvname; this.bexpr = bexpr;
	}
	
	public String genCode(Context con)
	{
		return "Bassign.(" + bvname.genCode(con) + "," + bexpr.genCode(con) + ")";
	}
	
	public int numIEs(Map<String,Integer> consts){return 0;}
	public int numBEs(Map<String,Integer> consts){return (bexpr.isConst() || bexpr.isVar()) ? 0 : 1;}
	
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) {return bexpr.printWithVals(ivals,bvals);}
	protected String getIE(int i, Map<String, Integer> ivals) //this should never get called
	{
		throw new ParserException("BAssign.getIE(i) called but BAssigns do not contain IEs");
	}
	
	public String toString() {return bvname + ":= " + bexpr;}
}	

interface BVName extends VName {}

class BVarName implements BVName
{
	String name;
	public BVarName(String name)
	{if (name==null) throw new ParserException("name argument to BVName null"); this.name = name;}
	
	public String genCode(Context con) 
	{ 
		return con.getBEncoding(name);
	}
	
	public String toString() { return name; }

}

class BArrayName implements BVName
{
	String name;
	IConst i;
	public BArrayName(String name, IConst i)
	{
		if (name==null) throw new ParserException("name argument to BArrayName null"); 
		if (i==null) throw new ParserException("index argument to BArrayName null");
		
		this.name = name; this.i = i;
	}

	public String genCode(Context con) 
	{ 
		return con.getBAEncoding(name) + "." + i.genCode(con);
	}
	
	public String toString() {return name + "[" + i + "]";}

}
	
class Sig extends aCmd
{
	String s;
	public Sig(String s) 
	{
		if (s==null) throw new ParserException("Argument to Sig null");
		this.s = s;
	}
	
	public String genCode(Context con)
	{
		return "Sig." + s;
	}
	
	protected String getIE(int i, Map<String, Integer> ivals) //this should never get called
	{
		throw new ParserException("Sig.getIE(i) called but Sigs do not contain IEs");
	}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) //this should never get called
	{
		throw new ParserException("Sig.getBE(i) called but Sigs do not contain BEs");
	}
	
	
	public int numIEs(Map<String,Integer> consts){return 0;}
	public int numBEs(Map<String,Integer> consts){return 0;}

	
	public String toString() { return "sig(" + s + ")";}
}
	
class ISig extends aCmd
{
	String s;
	IExpr i;
	public ISig(String s, IExpr i) 
	{
		if (s==null) throw new ParserException("s argument to ISig null");
		if (i==null) throw new ParserException("i argument to ISig null");
	
		this.s = s; this.i=i;
	}
	
	public String genCode(Context con)
	{
		return "ISig.(" + s + "," + i.genCode(con) + ")";
	}
	
	
	public int numIEs(Map<String,Integer> consts){return ( i.isVar() || i.isConst())? 0 : 1;}
	public int numBEs(Map<String,Integer> consts){return 0;}

	
	
	protected String getIE(int i, Map<String, Integer> ivals) 
	{
		return this.i.printWithVals(ivals);
	}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) //this should never get called
	{
		throw new ParserException("ISig.getBE(i) called but Sigs do not contain BEs");
	}
	
	public String toString() { return "isig(" + s + ", " + i + ")";}
}

class Atomic extends aCmd
{
	Cmd c;
	public Atomic(Cmd c) 
	{
		if (c==null) throw new ParserException("Argument to Atomic null");
		this.c = c;
	}
	
	public String genCode(Context con)
	{
		return "Atomic." + c.genCode(con);
	}
	
	
	public int numIEs(Map<String,Integer> consts){return c.numIEs(consts);}
	public int numBEs(Map<String,Integer> consts){return c.numBEs(consts);}
	
	protected String getIE(int i, Map<String, Integer> ivals) {return c.getIExpr(i, ivals);}
	protected String getBE(int i, Map<String, Integer> ivals, Map<String, Boolean> bvals) {return c.getBExpr(i,ivals,bvals);}
	
	public String toString() {return "atomic " + c;}
}












