import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Set;

public class Context implements Genable
{
	final boolean DEBUG = false;
	Vector<String> is,bs,ias,bas;//names of integer/boolean variables/arrays in order
	Collection<String> consts;//currently registered constants.  Will be replaced each time context is used by a different process (since constants are local to a process)
	Map<String, Integer> globalConsts;//the global constants


	Collection<String> sigs, isigs;
	Map<String, Integer> iinits;
	Map<String, Boolean> binits;
	Map<String, Vector<Integer>> iainits;
	Map<String, Vector<Boolean>> bainits;
	//Map<String, Integer> procs;// a mapping from the names of processes to the number of arguments they take
	Collection<String>procs;
	Collection<String>progs;//the names of programs

	public Context()
	{
		is = new Vector<String>();
		bas = new Vector<String>();
		ias = new Vector<String>();
		bs = new Vector<String>(); 
		consts = new Vector<String>();
		globalConsts = new TreeMap<String, Integer>();
		sigs = new Vector<String>();
		isigs = new Vector<String>();
		iinits = new TreeMap<String, Integer>();
		binits = new TreeMap<String, Boolean>();
		iainits = new TreeMap<String, Vector<Integer>>();
		bainits = new TreeMap<String, Vector<Boolean>>();
		//procs = new TreeMap<String, Integer>();
		procs = new Vector<String>();
		progs = new Vector<String>();
	}

	//methods for registering name as the appropriate type, with or without an initialization
	void newI(String name) {checkName(name);is.add(name);trace(name + " added to is");}
	void newB(String name) {checkName(name);bs.add(name);trace(name + " added to bs");}
	void newIA(String name) {checkName(name);ias.add(name);trace(name + " added to ias");}
	void newBA(String name) {checkName(name);bas.add(name);trace(name + " added to bas");}
	void newI(String name, int init) {checkName(name);is.add(name);iinits.put(name,init);trace(name + " added to is");}
	void newB(String name, boolean init) {checkName(name);bs.add(name);binits.put(name,init); trace(name + " added to bs");}
	void newIA(String name, Vector<Integer> init) {checkName(name);ias.add(name);iainits.put(name, init); trace(name + " added to ias");}
	void newBA(String name, Vector<Boolean>init) {checkName(name);bas.add(name);bainits.put(name, init); trace(name + " added to bas");}

	void newGlobConst(String name, int value){checkName(name); globalConsts.put(name, value); trace(name + " = " + value + " added to globalConsts");	}

	void newSig(String name) {checkName(name);sigs.add(name);trace(name + " added to sigs");}
	void newISig(String name) {checkName(name);isigs.add(name);trace(name + " added to isigs");}

	void newProcess(String name/*, int args*/) {checkName(name);procs.add(name);/*procs.put(name,args);*/}	
	void newProgram(String name){checkName(name);progs.add(name);}	

	void registerConstants(Collection<String> newConsts)  //replaces current list of integer constants with those passed in
	{
		deregisterConsts(); //ensure this.consts is empty before checking for clashes
		for (Iterator<String> i = newConsts.iterator(); i.hasNext(); )
		{
			String s = i.next();
			checkName(s);
			consts.add(s);
		}
	}
	
	void deregisterConsts() {consts.clear();trace("Constants deregistered");}

	void checkName(String name)
	{
		//checks to see if name is already registered in the context
		//this shouldnt really be needed since the lexer declares names it has already seen as the appropriate type
		trace("Checking if " + name + " has already been declared");
	
		if (is.contains(name)) {throw new ParserException(name + " declared but is already declared as an integer");}
		else if (bs.contains(name)) {throw new ParserException(name + " declared but is already declared as a boolean");}
		else if (ias.contains(name)) {throw new ParserException(name + " declared but is already declared as an integer array");}
		else if (bas.contains(name)) {throw new ParserException(name + " declared but is already declared as a boolean array");}
		else if (consts.contains(name)) {throw new ParserException(name + " declared but is already declared as a constant");}
		else if (globalConsts.containsKey(name)) {throw new ParserException(name + " declared but is already declared as a  global constant");}
		else if (sigs.contains(name)) {throw new ParserException(name + " declared but is already declared as a signal");}
		else if (isigs.contains(name)) {throw new ParserException(name + " declared but is already declared as an integer signal");}
		else if (procs.contains/*Key*/(name)) {throw new ParserException(name + " declared but is already declared as a process");}
		else if (progs.contains(name)) {throw new ParserException(name + " declared but is already declared as an program");}
		else if (name.equals("assertionfailed")) {throw new ParserException(name + " declared but assertionfailed is reserved for use by assert statements");}
		
	}

	//methods for retrieving the encoding of name. Throws an exception if name has not been declared for the appropriate type
	String getIEncoding(String name)
	{
		if (is.contains(name))
			return "I." + (is.indexOf(name) + 1);
		else
			throw new ParserException(name + " has not been declared as an int");
	}
	
	String getBEncoding(String name)
	{
		if (bs.contains(name))
			return "B." + (bs.indexOf(name) + 1);
		else
			throw new ParserException(name + " has not been declared as a boolean");
	}
	String getIAEncoding(String name)
	{
		if (ias.contains(name))
			return "IA." + (ias.indexOf(name) + 1);
		else
			throw new ParserException(name + " has not been declared as an int array");
	}
	String getBAEncoding(String name)
	{
		if (bas.contains(name))
			return "BA." + (bas.indexOf(name) + 1);
		else
			throw new ParserException(name + " has not been declared as an boolean array");
	}

	//int numArgs(String proc) {return procs.get(proc);}
	
	boolean isConstant(String name) {return (consts.contains(name) || globalConsts.containsKey(name));}
	boolean isI(String name) { return is.contains(name);}
	boolean isB(String name) { return bs.contains(name);}
	boolean isIA(String name) { return ias.contains(name);}
	boolean isBA(String name) { return bas.contains(name);}
	boolean isSig(String name) { return sigs.contains(name);}
	boolean isISig(String name) { return isigs.contains(name);}
	boolean isProcess(String name){return procs.contains/*Key*/(name);}
	boolean isProgram(String name) { return progs.contains(name);}

	private void trace (String s)
	{if (DEBUG) System.out.println("Context: " + s);}

	public String genCode(Context con){return genCode();}

	public String genCode()
	{
		String s = new String();
		
		
		//generate code for channels, but also for renaming them to channels tagged with the process number
		//name the tagged channels the same but with a t on the end.
		
		String cs = "assertionfailed"; //a comma seperated list of the signals
		String tcs = "assertionfailedT";//signals tagged with process number
		String acs = "at_assertionfailed";//signals tagged with at_
		String atcs = "at_assertionfailedT";//signals tagged with at_ and process number
		String renaming = "[[assertionfailed <- assertionfailedT.j, at_assertionfailed <- at_assertionfailedT.j";
		String atomic = "[[ assertionfailed <- at_assertionfailed";
		String unatomic = "[[ at_assertionfailedT.j <- assertionfailedT.j";
		
		{
		Iterator<String> i = sigs.iterator();
		while (i.hasNext())
		{
			String channel = i.next();
			cs = cs + ", " + channel;
			tcs = tcs + ", " + channel + "T";
			renaming = renaming + ", " + channel + " <- " + channel + "T.j, at_" + channel + " <- at_" + channel + "T.j";
			acs +=  ", at_" + channel;
			atcs +=  ", at_" + channel + "T";
			atomic += ", " + channel + " <- at_" + channel;
			unatomic += ", at_" + channel + "T.j <- " + channel + "T.j";
		
		}		
		s = s + "channel " + cs + ", " + acs + "\nSignals = {" + cs + "}\nchannel " + tcs + ", " + atcs + ":{0..nps-1}\ntaggedSignals = {" + tcs + "}\n" ;
		}
		if (isigs.size() > 0)
		{
			cs = new String(); //a comma seperated list of the signals
			tcs = new String();
			acs = new String();
			atcs = new String();
			
			Iterator<String> i = isigs.iterator();
			cs = i.next();
			tcs = cs + "T";
			acs = "at_" + cs;
			atcs = "at_" + cs + "T";
			renaming = renaming + ", " + cs + ".i <- " + cs + "T.i.j, at_" + cs + ".i <- at_" + cs + "T.i.j";
			atomic += ", " + cs + ".i <- at_" + cs + ".i";
			unatomic += ", at_" + cs + "T.i.j <- " + cs + "T.i.j";
			
			while (i.hasNext())
			{
				String channel = i.next();
				cs = cs + ", " + channel;
				tcs = tcs + ", " + channel + "T";
				renaming = renaming + ", " + channel + ".i <- " + channel + "T.i.j, at_" + channel + ".i <- at_" + channel + "T.i.j";
				acs +=  ", at_" + channel;
				atcs +=  ", at_" + channel + "T";
				atomic += ", " + channel + ".i <- at_" + channel + ".i";
				unatomic += ", at_" + channel + "T.i.j <- " + channel + "T.i.j";
			}
			s = s + "channel " + cs + ", " + acs + ":{MinI..MaxI}\nISignals = {" + cs + "}\nchannel " + tcs + ", " + atcs + ":{MinI..MaxI}.{0..nps-1}\ntaggedISignals = {" + tcs + "}\n";
		}
		else s = s + "ISignals = {}\ntaggedISignals={}\n";
		
		//s = s + "sigRenameAll(p) =  p" + renaming + " | i <-{MinI..MaxI}, j<-{0..nps-1} ]]\n";
		s = s + "sigRenameJ(p,j) =  p" + renaming + " | i <-{MinI..MaxI}]]\n";
		s += "atomicSigs(p) = p" + atomic + "| i <-{MinI..MaxI}]]\n";
		s += "unatomicSigs(p) = p" + unatomic + " | i <-{MinI..MaxI}, j<-{0..nps-1} ]]\n";
		
		
		
		s = s + "niv=" + (( is.size() > 0 )? is.size() : 1) + "\n";
		s = s + "nbv=" +(( bs.size() > 0 )? bs.size() : 1) + "\n";
		s = s + "nia=" + ias.size() + "\n";
		s = s + "nba=" + bas.size() + "\n\n";
		
		for (Iterator<String> i = globalConsts.keySet().iterator(); i.hasNext();)
		{
			String name = i.next();
			s = s + name + " = " + globalConsts.get(name) + "\n";
		}
		

		return s;	 
	}
	
	public String genInit()
	{
		String s = "(<";
		
		Iterator<String> i = iinits.keySet().iterator();
		while (i.hasNext())
		{
			String name = i.next();
			//if (iinits.containsKey(name))
			s = s + "(IVar." + getIEncoding(name) + "," + iinits.get(name) + "),";
		}
		i = iainits.keySet().iterator(); 
		while (i.hasNext())
		{
			String name = i.next();
			Vector<Integer> inits = iainits.get(name);
			
			int j = 0;
			while (j< inits.size())
			{
				s = s + "(IVar." + getIAEncoding(name) + "." + j + "," + inits.elementAt(j) + "),";
				j++;
			}
		}
		
		if (s.endsWith(",")) s = s.substring(0, s.length() -1);
		s = s + ">,<";
		
		i = bs.iterator();
		while (i.hasNext())
		{
			String name = i.next();
			if (binits.containsKey(name))
				s = s + "(BVar." + getBEncoding(name) + "," + binits.get(name) + "),";
		}
		i = bainits.keySet().iterator(); 
		while (i.hasNext())
		{
			String name = i.next();
			Vector<Boolean> inits = bainits.get(name);
			
			int j = 0;
			while (j< inits.size())
			{
				s = s + "(BVar." + getBAEncoding(name) + "." + j + "," + inits.elementAt(j) + "),";
				j++;
			}
		}

		if (s.endsWith(",")) s = s.substring(0, s.length() -1);
		s = s + ">)";
		return s;


		
	}
	
	public String getIName(int i) {return is.elementAt(i-1);}
	public String getBName(int i) {return bs.elementAt(i-1);}
	public String getIAName(int i) {return ias.elementAt(i-1);}
	public String getBAName(int i) {return bas.elementAt(i-1);}

}


