import java.util.Map;

public interface IConst extends Genable 
{
	public int constVal(Map<String, Integer> consts);//returns the value of the IConst
	
	public String printWithVals(Map<String, Integer> vals);

}

abstract class BinIConst implements IConst
{
	String op;
	IConst i1,i2;
	BinIConst(String op, IConst i1, IConst i2)
	{
		if (op==null) throw new ParserException("op Argument to BinIConst null");
		if (i1==null) throw new ParserException("i1 Argument to BinIConst (" + op +") null");
		if (i2==null) throw new ParserException("i2 Argument to BinIConst (" + op +")  null");
		this.op=op;
		this.i1=i1;
		this.i2=i2;
		
	}
	
	public String genCode(Context con) { return toString(); }

	public String toString()
	{ return "(" + i1 + op + i2 + ")";}
	
	public String printWithVals(Map<String, Integer> vals)
	{
		return i1.printWithVals(vals) + op + i2.printWithVals(vals);
	}
}

class AddConst extends BinIConst
{
	public AddConst(IConst i1, IConst i2){super("+",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) + i2.constVal(consts);}
}
class SubConst extends BinIConst
{
	public SubConst(IConst i1, IConst i2){super("-",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) - i2.constVal(consts);}
}
class MultConst extends BinIConst
{
	public MultConst(IConst i1, IConst i2){super("*",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) * i2.constVal(consts);}
}
class DivConst extends BinIConst
{
	public DivConst(IConst i1, IConst i2){super("/",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) / i2.constVal(consts);}
}
class ModConst extends BinIConst
{
	public ModConst(IConst i1, IConst i2){super("%",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) % i2.constVal(consts);}
}


class NegIConst implements IConst
{
	IConst i;
	NegIConst(IConst i)
	{
		if (i==null) throw new ParserException("i Argument to NegIConst null");
		this.i=i;
	}
	
	public String genCode(Context con) { return toString(); }

	public String toString()
	{ return "(-(" + i + "))";}
	
	public int constVal(Map<String, Integer> consts){return -i.constVal(consts);}
	public String printWithVals(Map<String, Integer> vals){return "-(" + i.printWithVals(vals) + ")";} 
	
}

class NumIConst implements IConst
{
	int i;
	NumIConst(int i)
	{
		this.i=i;
	}
	
	public String genCode(Context con) { return toString(); }

	public String toString()
	{ return String.valueOf(i);}
	
	public int constVal(Map<String, Integer> consts){return i;}
	public String printWithVals(Map<String, Integer> vals){return String.valueOf(i);} 
}

class VarIConst implements IConst
{
	String name;
	VarIConst(String name)
	{
		if (name == null) throw new ParserException("name argument to VarIConst null");
		this.name=name;
	}
	
	public String genCode(Context con) { return toString(); }

	public String toString()
	{ return name;}
	
	
	public int constVal(Map<String, Integer> consts)
	{
		if (consts.containsKey(name)) return consts.get(name);
		else throw new InterpreterException("VarIConst.constVal(consts) - " + name + " not in consts");
	
	}
	
	public String printWithVals(Map<String, Integer> vals)
	{	
		if (vals.containsKey(name)) return name + "(" + vals.get(name) + ")";
		else throw new InterpreterException("VarIConst.printWithVal(consts) - " + name + " not in consts");
	} 
	
	
}