import java.util.Map;

public interface IExpr extends Genable 
{ 
	public boolean isConst();
	public boolean isVar();
	public int constVal(Map<String, Integer> consts);//returns the value of the IExpr. Throws an ezxception if not a constant
	
	public String printWithVals(Map<String, Integer> vals);
	
	static final boolean DEBUG = true;
}

abstract class BinIExpr implements IExpr
{
	String op, print;
	IExpr i1,i2;
	BinIExpr(String op, String print, IExpr i1, IExpr i2)
	{
		if (op==null) throw new ParserException("op Argument to BinIExp null");
		if (print==null) throw new ParserException("print Argument to BinIExpr(" + op +") null");
		if (i1==null) throw new ParserException("i1 Argument to BinIExp (" + op +") null");
		if (i2==null) throw new ParserException("i2 Argument to BinIExp (" + op +")  null");
		this.op=op;
		this.print = print;
		this.i1=i1;
		this.i2=i2;
		
	}
	
	public String genCode(Context con) {return op + "." + i1.genCode(con) + "." + i2.genCode(con); }

	
	public String toString()
	{ return i1 + print + i2;}
	
	public String printWithVals(Map<String, Integer> vals)
	{
		return i1.printWithVals(vals) + print + i2.printWithVals(vals);
	}
	
	
	public boolean isConst() {return i1.isConst() && i2.isConst();}
	public boolean isVar() {return false;}

}

class Plus extends BinIExpr
{
	public Plus(IExpr i1, IExpr i2){super("Plus","+",i1,i2);}
	
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) + i2.constVal(consts);}
}
class Minus extends BinIExpr
{
	public Minus(IExpr i1, IExpr i2){super("Minus","-",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) - i2.constVal(consts);}
}
class Times extends BinIExpr
{
	public Times(IExpr i1, IExpr i2){super("Times","*",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) * i2.constVal(consts);}
}
class Div extends BinIExpr
{
	public Div(IExpr i1, IExpr i2){super("Div","/",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) / i2.constVal(consts);}
}
class Mod extends BinIExpr
{
	public Mod(IExpr i1, IExpr i2){super("Mod","%",i1,i2);}
	public int constVal(Map<String, Integer> consts){return i1.constVal(consts) % i2.constVal(consts);}
}

class Neg implements IExpr
{
	IExpr i;
	public Neg(IExpr i)
	{if (i==null) throw new ParserException("Argument to Neg null");this.i =i;}
	
	public String genCode(Context con) { return "Minus.Const.0." + i.genCode(con);}

	
	public String toString()
	{ return "-(" + i + ")";}
	
	public String printWithVals(Map<String, Integer> vals)
	{
		return "-(" + i.printWithVals(vals) + ")";
	}
	
	public boolean isConst() {return i.isConst();}
	public boolean isVar() {return false;}
	public int constVal(Map<String, Integer> consts){return -i.constVal(consts);}
}

class IVar implements IExpr
{
	String name;
	public IVar(String name){if (name==null) throw new ParserException("Argument to IVar null");this.name = name;}
	
	public String genCode(Context con) 
	{ 
		/*if (con.isConstant(name) 	//was possible for this to be a constant.  No longer the case
			return name;
		else*/
			return "IVar." + con.getIEncoding(name);
	}

	public String toString() {return name;}
	public boolean isConst() {return false;}
	public boolean isVar() {return true;}
	public int constVal(Map<String, Integer> consts){throw new ParserException("constVal(consts) called for an expression involving a variable");}
	
	public String printWithVals(Map<String, Integer> vals)
	{
		if (vals.containsKey(name)) return name + "(" + vals.get(name) + ")";
		else return name;
	}
}

class IArray implements IExpr
{
	String name;
	IConst i;
	public IArray(String name, IConst i)
	{
		if (name==null) throw new ParserException("name argument to IArray null"); 
		if (i==null) throw new ParserException("index argument to IArray null"); 
		
		this.name = name; this.i = i;
	}

	public String genCode(Context con) 
	{ 
		return "IVar." + con.getIAEncoding(name) + "." + i.genCode(con);
	}

	public String toString() {return name + "[" + i + "]";}
	public String printWithVals(Map<String, Integer> vals)
	{
		String me = name + "[" + i.constVal(vals) + "]";
	
		//trace("me = " + me);
		//trace("vals = " + vals);
		
		if (vals.containsKey(me)) return name + "[" + i.printWithVals(vals) + "](" + vals.get(me) + ")";
		else return name + "[" + i.printWithVals(vals) + "]";
	}
	
	
	public boolean isConst() {return false;}
	public boolean isVar() {return true;}
	public int constVal(Map<String, Integer> consts){throw new ParserException("constVal(consts) called for an expression involving a variable");}

	void trace(String s) {if (DEBUG) System.out.println("IArray: " + s);}
}

/*class IExprConst implements IExpr //wraps a constant into an expression
{
	IConst i;
	public IExprConst(IConst i)
	{
		if (i == null) throw new ParserException("i argument to IExprConst null");
		this.i = i;
	}
	
	public String genCode(Context con) {return "Const." + i.genCode(con);}
	public String toString() {return i.toString();}
	
	public boolean isConst() {return true;}
	public boolean isVar() {return false;}
}*/

class NumConst implements IExpr
{
	int i;
	public NumConst(int i){this.i = i;}
	
	public String genCode(Context con) {return "Const." + String.valueOf(i);}

	public String toString() {return String.valueOf(i);}
	public String printWithVals(Map<String, Integer> vals)
	{
		return toString();
	}
	
	
	public boolean isConst() {return true;}
	public boolean isVar() {return false;}
	public int constVal(Map<String, Integer> consts){return i;}
}

class IVarConst implements IExpr //a process parameter
{
	String name;
	public IVarConst(String name)
	{if (name==null) throw new ParserException("name argument to IVarConst null");this.name = name;}
	
	public String genCode(Context con) 
	{ 
		//this check should no longer be needed since lexer now only returns CONST if name is a constant
		//but it doesnt hurt much to leave it in
		if (con.isConstant(name))
			return "Const." + name;
		else
			throw new ParserException ( name + " expected to be a constant" );
	}
	public String toString() {return name;}
	
	public boolean isConst() {return true;}
	public boolean isVar() {return false;}
	
	public int constVal(Map<String, Integer> consts)
	{
		if (consts.containsKey(name)) return consts.get(name).intValue();
		else throw new ParserException("IVarConst.constVal(consts) - consts does not contain " + name);
	}
	
	public String printWithVals(Map<String, Integer> vals)
	{
		if (vals.containsKey(name)) return name + "(" + vals.get(name) + ")";
		else return name;
	}
}


class Max implements IExpr
{
	IExpr i1,i2;
	Max(IExpr i1, IExpr i2)
	{
		if (i1==null) throw new ParserException("i1 Argument to Max null");
		if (i2==null) throw new ParserException("i2 Argument to Max null");
		this.i1=i1;
		this.i2=i2;
		
	}
	
	public String genCode(Context con) {return "Max." + i1.genCode(con) + "." + i2.genCode(con); }

	
	public String toString()
	{ return "max(" + i1 + "," + i2 + ")";}
	
	public String printWithVals(Map<String, Integer> vals)
	{
		return "max(" + i1.printWithVals(vals) + "," + i2.printWithVals(vals) + ")";
	}
	
	
	public boolean isConst() {return i1.isConst() && i2.isConst();}
	public boolean isVar() {return false;}
	
	public int constVal(Map<String, Integer> consts)
	{
		return ( i1.constVal(consts) >= i2.constVal(consts)) ? i1.constVal(consts) : i2.constVal(consts);	
	}

}


class Min implements IExpr
{
	IExpr i1,i2;
	Min(IExpr i1, IExpr i2)
	{
		if (i1==null) throw new ParserException("i1 Argument to Min null");
		if (i2==null) throw new ParserException("i2 Argument to Min null");
		this.i1=i1;
		this.i2=i2;
		
	}
	
	public String genCode(Context con) {return "Min." + i1.genCode(con) + "." + i2.genCode(con); }

	
	public String toString()
	{ return "min(" + i1 + "," + i2 + ")";}
	
	public String printWithVals(Map<String, Integer> vals)
	{
		return "min(" + i1.printWithVals(vals) + "," + i2.printWithVals(vals) + ")";
	}
	
	public int constVal(Map<String, Integer> consts)
	{
		return ( i1.constVal(consts) <= i2.constVal(consts)) ? i1.constVal(consts) : i2.constVal(consts);	
	}
	
	public boolean isConst() {return i1.isConst() && i2.isConst();}
	public boolean isVar() {return false;}

}