import java.awt.*;
import java.awt.event.*; 
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.io.PrintStream;
import java.io.FileReader;
import java.io.BufferedReader;
import java.util.Vector;
import java.util.Iterator;
 
public class InterpreterGUI 
{
    	
    	final static boolean DEBUG = false;
    	
    	final static boolean INITIAL_SHOW_VARS = false;
    	final static boolean INITIAL_SHOW_EXPRS = true;
    	final static boolean INITIAL_SHOW_ATOMICS = true;
    	
    	final static String CSPFILE = "output.csp";
    	final static String DEFAULTRESULTSFILE = "results";
    	String resultsFile = null;
    	
    	//declerations of all the components to be used
    	protected JFrame frame = new JFrame();
    	protected TableData data = new TableData(new Vector<String>(), new Vector<TableDataInterpreter.Entry>(),INITIAL_SHOW_VARS, INITIAL_SHOW_EXPRS,INITIAL_SHOW_ATOMICS);
    	protected JList assertsList = new JList();	
    	protected JLabel result = new JLabel("Result:");
	protected JScrollPane assertsPane = new JScrollPane(assertsList);
	protected JCheckBox vars = new JCheckBox("Show Variable Reads");
	protected JCheckBox exps = new JCheckBox("Show Expression Evaluation Starts");
	protected JCheckBox atomic = new JCheckBox("Show Atomic Sections");
	protected JButton runFDR = new JButton("Run FDR");
	//protected JButton interrupt = new JButton("Interrupt"); removed this button to halt progress of long processes since it did not actually stop fdr from running
	protected JTable counterExample = new JTable(data);
	protected JScrollPane examplePane = new JScrollPane(counterExample);
	protected JMenuBar menubar = new JMenuBar();
	protected JMenu fileMenu= new JMenu("File");
	protected JMenuItem load = new JMenuItem("Load");
	protected JMenuItem reload = new JMenuItem("Reload");
	protected JMenuItem loadResults = new JMenuItem("Load Results");
	protected JMenuItem saveResults = new JMenuItem("Save Results");
	protected JFileChooser fileChooser = new JFileChooser();
    	
    	
    	protected Vector<TableDataInterpreter.AssertionOutput> results = new Vector<TableDataInterpreter.AssertionOutput>();
    	
    	
    	protected String fileName = "";
    	
    	protected Interpreter interpreter;
    	
    	public void addComponentsToFrame() 
    	{
    		Container pane = frame.getContentPane();

	//behaviour of components

    		//JList assertsList
		
		assertsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		assertsList.addListSelectionListener( 
			new ListSelectionListener()
			{
				int current;
				public void valueChanged(ListSelectionEvent e) 
				{
					if (assertsList.getSelectedIndex() == -1) 
					{
						data.setData(new Vector<String>(), new Vector<TableDataInterpreter.Entry>());
						result.setText("Result:");
					} 
					else
					{
						TableDataInterpreter.AssertionOutput selected = results.elementAt(assertsList.getSelectedIndex());
					
						data.setData(selected.procs,selected.counterExample);
						result.setText("Result: " + selected.result);
					}
					setTableWidths(counterExample);
				}
			});
		
		//JScrollPane assertsPane
		
		
		//JLabel result
		
		
		//JCheckBox vars
		
		vars.setSelected(INITIAL_SHOW_VARS);
		vars.addItemListener(
			new ItemListener()
			{
				public void itemStateChanged(ItemEvent e)
				{
					data.changeVarReads(e.getStateChange() == ItemEvent.SELECTED);
				}
			});
		
		//JCheckBox exps
		
		exps.setSelected(INITIAL_SHOW_EXPRS);
		exps.addItemListener(
			new ItemListener()
			{
				public void itemStateChanged(ItemEvent e)
				{
					data.changeExprStarts(e.getStateChange() == ItemEvent.SELECTED);
				}
			});
		
		//JCheckBox atomic
		
		atomic.setSelected(INITIAL_SHOW_ATOMICS);
		atomic.addItemListener(
			new ItemListener()
			{
				public void itemStateChanged(ItemEvent e)
				{
					data.changeAtomics(e.getStateChange() == ItemEvent.SELECTED);
				}
			});
		
		//JButton runFDR
		
		runFDR.setEnabled(false);
		runFDR.addActionListener( new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							runFDR();
						}
					});
		
		//JButton interrupt
		//interrupt.setEnabled(false);
		
		//JTable counterExample
		
		counterExample.setColumnSelectionAllowed(false);
		counterExample.setRowSelectionAllowed(false);
		
		//JScrollPane examplePane

		//JMenuBar menubar
		
		//JMenu fileMenu
		
		//JMenuItem load
		
		load.addActionListener( new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							int returnVal = fileChooser.showOpenDialog(load);	
							
							if (returnVal == JFileChooser.APPROVE_OPTION) 
								load(fileChooser.getSelectedFile().toString());
						}
					});
		
		//JMenuItem reload 
		
		reload.addActionListener( new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							load(fileName);
						}
					});
		reload.setEnabled(false);
		
		//JMenuItem loadResults
		
		loadResults.addActionListener( new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							int returnVal = fileChooser.showOpenDialog(loadResults);	
							
							if (returnVal == JFileChooser.APPROVE_OPTION) 
								loadResults(fileChooser.getSelectedFile().toString());
						}
					});
					
		loadResults.setEnabled(false);
		
		//JMenuItem saveResults
		
		saveResults.addActionListener( new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							int returnVal = fileChooser.showSaveDialog(saveResults);	
							
							if (returnVal == JFileChooser.APPROVE_OPTION) 
								saveResults(fileChooser.getSelectedFile().toString());
						}
					});
		
			
		saveResults.setEnabled(false);
		
		//JFileChooser fileChooser

	//Layout of components

		pane.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor=GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;

		
    		//JList assertsList
		
		//JScrollPane assertsPane
		
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = .5;
		c.gridheight = 5;
		pane.add(assertsPane, c);
		c.gridheight = 1;//reset
		c.weightx = 0;
		c.weighty = 0;
		
		//JLabel result
		
		result.setFont(result.getFont().deriveFont(20.0f));
		c.gridx = 0;
		c.gridy = 5;
		c.gridwidth= 2;
		pane.add(result, c);
		c.gridwidth = 1;
		
		//JCheckBox vars
		
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		pane.add(vars, c);
		
		//JCheckBox exps
		
		c.gridx = 1;
		c.gridy = 1;
		pane.add(exps, c);
		
		//JCheckBox atomic
		
		c.gridx = 1;
		c.gridy = 2;
		pane.add(atomic, c);
		
		//JButton runFDR
		c.gridx = 1;
		c.gridy = 3;
		pane.add(runFDR,c);
		
		//JButton interrupt
		/*c.gridx = 1;
		c.gridy = 4;
		pane.add(interrupt,c);*/
		
		//JTable counterExample
		
		counterExample.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		counterExample.setEnabled(false);
		//counterExample.setPreferredScrollableViewportSize(new Dimension(500, 700));
		
		//JScrollPane examplePane

		c.gridwidth = 2;
		c.gridx = 0;
		c.gridy = 6;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		pane.add(examplePane, c);
		
		//JMenuBar menubar
		
		frame.setJMenuBar(menubar);

		//JMenu fileMenu
		
		menubar.add(fileMenu);		
		
		//JMenuItem load
		
		fileMenu.add(load);
		
		//JMenuItem reload 
		
		fileMenu.add(reload);
		
		//JMenuItem loadResults
		
		fileMenu.add(loadResults);
		
		//JMenuItem saveResults
		
		fileMenu.add(saveResults);
		
    	}

	
	//modified from TableRenderDemo
	static protected void setTableWidths(JTable table) 
	{
		TableModel model = table.getModel();
		TableColumn column = null;
		Component comp = null;
		int headerWidth = 0;
		int cellWidth = 0;
		//Object[] longValues = model.longValues;
		
		TableCellRenderer headerRenderer =
		table.getTableHeader().getDefaultRenderer();
	
		for (int i = 0; i < table.getColumnCount(); i++) 
		{
				
			column = table.getColumnModel().getColumn(i);
		
			comp = headerRenderer.getTableCellRendererComponent(
						null, column.getHeaderValue(),
						false, false, 0, 0);
			headerWidth = comp.getPreferredSize().width;
			cellWidth = 90;//have a minimum width of 100
		
			for(int j = 0; j < table.getRowCount(); j++)
			{
				comp = table.getDefaultRenderer(model.getColumnClass(i)).
					getTableCellRendererComponent(
						table, table.getValueAt(j,i),
						false, false, 0, i);
				cellWidth = Math.max(cellWidth,comp.getPreferredSize().width);
			}
		
			trace("Initializing width of column " + i + ". " + "headerWidth = " + headerWidth + "; cellWidth = " + cellWidth);
		
		
			//XXX: Before Swing 1.1 Beta 2, use setMinWidth instead.
			column.setPreferredWidth(Math.max(headerWidth, cellWidth)+10);
			//column.setMinWidth(100);//for empty columns
			//column.setResizable(false);
		}
    }


	/**
	* Create the GUI and show it.  For thread safety,
	* this method should be invoked from the
	* event-dispatching thread.
	*/
	private void createAndShowGUI() 
	{
		//Create and set up the window.
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
		//Set up the content pane.
		addComponentsToFrame();
	
		//Display the window.
		frame.pack();
		frame.setVisible(true);
	}

	public void run()
	{
		//Schedule a job for the event-dispatching thread:
		//creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
		public void run() {
			createAndShowGUI();
		}
		});
	}
	
	public void run(String source)
	{ run(source,CSPFILE,null,null);}
	
	public void run(final String source,final String CSPFile,final String readFDRFile,final String writeFDRFile) 
	{
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
		public void run() {
			createAndShowGUI();
			load(source, CSPFile, readFDRFile, writeFDRFile);
		}
		});
	
	}
	
	/**
	* For thread safety,
	* this method should be invoked from the
	* event-dispatching thread.
	*/
	protected void load(final String source)
	{
		//show busy cursor
		frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
		//change title
		frame.setTitle("Loading " + source);	
		fileName = source;
			
		assertsList.setEnabled(false);
		vars.setEnabled(false);
		exps.setEnabled(false);
		atomic.setEnabled(false);
		runFDR.setEnabled(false);
		load.setEnabled(false);
		reload.setEnabled(false);
		loadResults.setEnabled(false);
		saveResults.setEnabled(false);
	
		final SwingWorker worker = new SwingWorker<String[], Void>()
		{
			@Override
    			public String[] doInBackground()
    			{
     				try
				{
					resultsFile = null;	
					interpreter = new Interpreter(source);
					return new String[] {};
				}
				catch(java.io.FileNotFoundException e)
				{
					return new String[] {"File Not Found", e.getMessage()};
				}/*
				catch(java.io.IOException e)
				{
					return new String[] {"Error", e.getMessage()};
				}
				catch(InterruptedException e)
				{
					return new String[] {"Interrupted", e.getMessage()};
				}
				catch(ParserException e)
				{
					return new String[] {"Error Parsing File", e.getMessage()};
				}*/
			}
			
			@Override
    			public void done()
    			{
    				String[] message;
    				try
    				{
    					//if (isCancelled())
    					//	message = new String[] {"Interrupted", "Loading File Halted"};
    					//else
    					//{
					message = get();
					if (message.length == 0)
					{	//success
						Vector<String> asserts = new Vector<String>();
						Iterator<Assertion> i = interpreter.parser.asserts.iterator(); 
						while (i.hasNext())
							asserts.add(i.next().toString());
						assertsList.setListData(asserts);
						//assertsList.setSelectedIndex(-1);//DOES THIS WORK?? not needed anymore?
						
						//reenable options to get results or a new file
						
						runFDR.setEnabled(true);
						load.setEnabled(true);
						reload.setEnabled(true);
						loadResults.setEnabled(true);
						//interrupt.setEnabled(false);
						//for (ActionListener al : interrupt.getActionListeners())
						//	interrupt.removeActionListener(al);
							
						//set title
						frame.setTitle(fileName);
						
						//remove busy cursor
						frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						return;
					}
					//}
				}
				catch(InterruptedException e)
				{
					message = new String[] {"Interrupted", e.getMessage()};
				}
				catch(java.util.concurrent.ExecutionException e)
				{
					message = new String[] {"Error", e.getMessage()};
				}
    				//else error!
    				//{
    					JOptionPane.showMessageDialog(frame, message[1], message[0], JOptionPane.WARNING_MESSAGE);
					
					//clear everything
					data.setData(new Vector<String>(), new Vector<TableDataInterpreter.Entry>());
					assertsList.setListData(new String[] {});
					//assertsList.setEnabled(true);
					//vars.setEnabled(true);
					//exps.setEnabled(true);
					//atomic.setEnabled(true);
					load.setEnabled(true);
					reload.setEnabled(true);
					//interrupt.setEnabled(false);
					//for (ActionListener al : interrupt.getActionListeners())
					//	interrupt.removeActionListener(al);
					
					//set title
					frame.setTitle("Error Loading " + fileName);
					
					//remove busy cursor
					frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
					
					//remove loading screen
    				//}
    			}
		
		};
		
		
		/*interrupt.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							worker.cancel(true);
						}
					});
		interrupt.setEnabled(true);*/
		worker.execute();
	
	}
	
	
	/**
	* For thread safety,
	* this method should be invoked from the
	* event-dispatching thread.
	*/
	protected void runFDR()
	{
		//show busy cursor
		frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
		//change title
		frame.setTitle("Running FDR - " + fileName);	
			
		//disable everything
		assertsList.setEnabled(false);
		vars.setEnabled(false);
		exps.setEnabled(false);
		atomic.setEnabled(false);
		runFDR.setEnabled(false);
		load.setEnabled(false);
		reload.setEnabled(false);
		loadResults.setEnabled(false);
		saveResults.setEnabled(false);
	
		final SwingWorker worker = new SwingWorker<String[], Void>()
		{
			protected Vector<TableDataInterpreter.AssertionOutput> newResults;
			@Override
    			public String[] doInBackground()
    			{
     				try
				{
					//store results in RESULTSFILE so we can save them elsewhere if the
					//user requests it later on
					interpreter.runFDR(CSPFILE, DEFAULTRESULTSFILE);		
					TableDataInterpreter output = new TableDataInterpreter(interpreter.parser.context.globalConsts);
					interpreter.interpret(output);	
					
					newResults = output.getResults();
					resultsFile = DEFAULTRESULTSFILE;
					return new String[] {};
				}
				/*catch(java.io.FileNotFoundException e)
				{
					return new String[] {"File Not Found", e.getMessage()};
				}*/
				catch(java.io.IOException e)
				{
					return new String[] {"I/O Error", e.getMessage()};
				}
				catch(InterruptedException e)
				{
					return new String[] {"Interrupted", e.getMessage()};
				}/*
				catch(ParserException e)
				{
					return new String[] {"Error Parsing File", e.getMessage()};
				}
				catch(InterpreterException e)
				{
					return new String[] {"Error Interpreting Results", e.getMessage()};
				}
				catch(Throwable e)
				{
					return new String[] {"Error", e.getMessage()};
				}*/
			}
			
			@Override
    			public void done()
    			{
    				String[] message;
    				try
    				{
    					//if (isCancelled())
    					//	message = new String[] {"Interrupted", "FDR Halted"};
    					//else
    					//{ 
					message = get();
					if (message.length == 0)
					{//success
						results = newResults;
						
						//reenable everything
						assertsList.setEnabled(true);
						vars.setEnabled(true);
						exps.setEnabled(true);
						atomic.setEnabled(true);
						//runFDR.setEnabled(false);
						load.setEnabled(true);
						reload.setEnabled(true);
						//loadResults.setEnabled(false);
						saveResults.setEnabled(true);
						//interrupt.setEnabled(false);
						//for (ActionListener al : interrupt.getActionListeners())
						//	interrupt.removeActionListener(al);
						
						//set title
						frame.setTitle(fileName);
						
						//remove busy cursor
						frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						return;
						
						//remove loading screen
					}
					//}
				}
				catch(InterruptedException e)
				{
					message = new String[] {"Interrupted", e.getMessage()};
				}
				catch(java.util.concurrent.ExecutionException e)
				{
					message = new String[] {"Error", e.getMessage()};
				}
				/*catch(Throwable e)
				{
					message = new String[] {"Error", e.getMessage()};
				}*/
    				//else error!
    				//{
    					JOptionPane.showMessageDialog(frame, message[1], message[0], JOptionPane.WARNING_MESSAGE);
					
					//clear everything
					//data.setData(new Vector<String>(), new Vector<TableDataInterpreter.Entry>());
					//assertsList.setListData(new String[] {});
					//assertsList.setEnabled(true);
					//vars.setEnabled(true);
					//exps.setEnabled(true);
					//atomic.setEnabled(true);
					runFDR.setEnabled(true);
					load.setEnabled(true);
					reload.setEnabled(true);
					loadResults.setEnabled(true);
					//interrupt.setEnabled(false);
					//for (ActionListener al : interrupt.getActionListeners())
					//		interrupt.removeActionListener(al);
					
					//set title
					frame.setTitle("Error Running " + fileName);
					
					resultsFile = null;
					
					//remove busy cursor
					frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
					
					//remove loading screen
    				//}
    			}
		
		};
		
		/*interrupt.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							worker.cancel(true);
						}
					});
		interrupt.setEnabled(true);*/
		
		worker.execute();
		
	}
	
	
	/**
	* For thread safety,
	* this method should be invoked from the
	* event-dispatching thread.
	*/
	protected void loadResults(final String file)
	{
		//show busy cursor
		frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
		//change title
		frame.setTitle("Loading results for " + fileName + " from " + file);	
			
		//disable everything
		assertsList.setEnabled(false);
		vars.setEnabled(false);
		exps.setEnabled(false);
		atomic.setEnabled(false);
		runFDR.setEnabled(false);
		load.setEnabled(false);
		reload.setEnabled(false);
		loadResults.setEnabled(false);
		saveResults.setEnabled(false);
	
		final SwingWorker worker = new SwingWorker<String[], Void>()
		{
			protected Vector<TableDataInterpreter.AssertionOutput> newResults;
			@Override
    			public String[] doInBackground()
    			{
     				try
				{
					interpreter.loadOutput(file);		
					TableDataInterpreter output = new TableDataInterpreter(interpreter.parser.context.globalConsts);
					interpreter.interpret(output);	
					
					newResults = output.getResults();
					resultsFile = file;
					return new String[] {};
				}
				/*catch(java.io.FileNotFoundException e)
				{
					return new String[] {"File Not Found", e.getMessage()};
				}*/
				catch(java.io.IOException e)
				{
					return new String[] {"Error", e.getMessage()};
				}/*
				catch(InterruptedException e)
				{
					return new String[] {"Interrupted", e.getMessage()};
				}
				catch(ParserException e)
				{
					return new String[] {"Error Parsing File", e.getMessage()};
				}
				catch(InterpreterException e)
				{
					return new String[] {"Error Interpreting Results", e.getMessage()};
				}*/
			
			}
			
			@Override
    			public void done()
    			{
    				String[] message;
    				try
    				{
    					//if (isCancelled())
    					//	message = new String[] {"Interrupted", "Loading Results Halted"};
    					//else
    					//{
					message = get();
					if (message.length == 0)
					{//success
						results = newResults;
						
						//reenable everything
						assertsList.setEnabled(true);
						vars.setEnabled(true);
						exps.setEnabled(true);
						atomic.setEnabled(true);
						//runFDR.setEnabled(false);
						load.setEnabled(true);
						reload.setEnabled(true);
						//loadResults.setEnabled(false);
						saveResults.setEnabled(true);
						//interrupt.setEnabled(false);
						//for (ActionListener al : interrupt.getActionListeners())
						//	interrupt.removeActionListener(al);
							
						//set title
						frame.setTitle(fileName);
						
						//remove busy cursor
						frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						return;
						
						//remove loading screen
					}
					//}
				}
				catch(InterruptedException e)
				{
					message = new String[] {"Interrupted", e.getMessage()};
				}
				catch(java.util.concurrent.ExecutionException e)
				{
					message = new String[] {"Error", e.getMessage()};
				}
    				//else error!
    				//{
    					JOptionPane.showMessageDialog(frame, message[1], message[0], JOptionPane.WARNING_MESSAGE);
					
					//clear everything
					//data.setData(new Vector<String>(), new Vector<TableDataInterpreter.Entry>());
					//assertsList.setListData(new String[] {});
					//assertsList.setEnabled(true);
					//vars.setEnabled(true);
					//exps.setEnabled(true);
					//atomic.setEnabled(true);
					runFDR.setEnabled(true);
					load.setEnabled(true);
					reload.setEnabled(true);
					loadResults.setEnabled(true);
					//interrupt.setEnabled(false);
					//for (ActionListener al : interrupt.getActionListeners())
					//		interrupt.removeActionListener(al);
								
					//set title
					frame.setTitle("Error Loading Results for " + fileName);
					
					resultsFile = null;
					//remove busy cursor
					frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
					
					//remove loading screen
    				//}
    			}
		
		};
		
		/*interrupt.addActionListener(new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							worker.cancel(true);
						}
					});
		interrupt.setEnabled(true);*/
		
		worker.execute();
		
	}
	
	/**
	* For thread safety,
	* this method should be invoked from the
	* event-dispatching thread.
	*/
	protected void saveResults(final String file)
	{
		SwingWorker worker = new SwingWorker<String[], Void>()
		{
			@Override
    			public String[] doInBackground()
    			{
     				try
				{
					BufferedReader readFrom = new BufferedReader(new FileReader(resultsFile));
					PrintStream writeTo = new PrintStream(file);
					
					while (readFrom.ready())
						writeTo.println(readFrom.readLine());	
					
					return new String[] {};
				}
				catch(java.io.FileNotFoundException e)
				{
					return new String[] {"File Not Found", e.getMessage()};
				}
				catch(java.io.IOException e)
				{
					return new String[] {"I/O Error", e.getMessage()};
				}/*
				catch(InterruptedException e)
				{
					return new String[] {"Interrupted", e.getMessage()};
				}
				catch(ParserException e)
				{
					return new String[] {"Error Parsing File", e.getMessage()};
				}
				catch(InterpreterException e)
				{
					return new String[] {"Error Interpreting Results", e.getMessage()};
				}*/
			}
			
			@Override
    			public void done()
    			{
    				String[] message;
    				try
    				{
					message = get();
					if (message.length == 0)
						return;
				}
				catch(InterruptedException e)
				{
					message = new String[] {"Interrupted", e.getMessage()};
				}
				catch (java.util.concurrent.ExecutionException e)
				{
					message = new String[] {"Error", e.getMessage()};	
				}
				JOptionPane.showMessageDialog(frame, message[1], message[0], JOptionPane.WARNING_MESSAGE);
    			}
		
		};
		
		worker.execute();
		
	}
	
	
	/**
	* For thread safety,
	* this method should be invoked from the
	* event-dispatching thread.
	*/
	protected void load(final String source,final String CSPFile,final String readFDRFile,final String writeFDRFile)
	{
		//show loading screen?
		
		//show busy cursor
		frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
		//change title
		frame.setTitle("Loading " + source);	
		fileName = source;
			
		//clear data	
		/*data.setData(new Vector<String>(), new Vector<TableDataInterpreter.Entry>());
		assertsList.setListData(new Vector<String>());
		results = new Vector<TableDataInterpreter.AssertionOutput>();
		assertsList.setSelectedIndex(-1);//DOES THIS WORK??
		*/
		//disable everything
		assertsList.setEnabled(false);
		vars.setEnabled(false);
		exps.setEnabled(false);
		atomic.setEnabled(false);
		runFDR.setEnabled(false);
		load.setEnabled(false);
		reload.setEnabled(false);
		loadResults.setEnabled(false);
		saveResults.setEnabled(false);
	
		SwingWorker worker = new SwingWorker<String[], Void>()
		{
			protected Vector<TableDataInterpreter.AssertionOutput> newResults;
			@Override
    			public String[] doInBackground()
    			{
     				try
				{
					interpreter = new Interpreter(source, CSPFile, readFDRFile, writeFDRFile);		
					TableDataInterpreter output = new TableDataInterpreter(interpreter.parser.context.globalConsts);
					interpreter.interpret(output);	
					newResults = output.getResults();
					resultsFile = writeFDRFile;
					return new String[] {};
				}
				catch(java.io.FileNotFoundException e)
				{
					return new String[] {"File Not Found", e.getMessage()};
				}
				catch(java.io.IOException e)
				{
					return new String[] {"Error", e.getMessage()};
				}
				catch(InterruptedException e)
				{
					return new String[] {"Interrupted", e.getMessage()};
				}
				catch(ParserException e)
				{
					return new String[] {"Error Parsing File", e.getMessage()};
				}
				catch(InterpreterException e)
				{
					return new String[] {"Error Interpreting Results", e.getMessage()};
				}
			}
			
			@Override
    			public void done()
    			{
    				String[] message;
    				try
    				{
    					message = get();
					if (message.length == 0)
					{//success
						results = newResults;
						
						Vector<String> asserts = new Vector<String>();
						Iterator<TableDataInterpreter.AssertionOutput> i = results.iterator(); 
						while (i.hasNext())
							asserts.add(i.next().assertion);
						assertsList.setListData(asserts);
						//assertsList.setSelectedIndex(-1);//DOES THIS WORK?? not needed anymore?
						
						//reenable everything
						
						assertsList.setEnabled(true);
						vars.setEnabled(true);
						exps.setEnabled(true);
						atomic.setEnabled(true);
						runFDR.setEnabled(true);
						load.setEnabled(true);
						reload.setEnabled(true);
						loadResults.setEnabled(true);
						
						//set title
						frame.setTitle(fileName);
						
						//remove busy cursor
						frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						return;
						
						//remove loading screen
					}
				}
				catch(InterruptedException e)
				{
					message = new String[] {"Interrupted", e.getMessage()};
				}
				catch(java.util.concurrent.ExecutionException e)
				{
					message = new String[] {"Error", e.getMessage()};
				}
    				//else error!
    				//{
    					JOptionPane.showMessageDialog(frame, message[1], message[0], JOptionPane.WARNING_MESSAGE);
					
					//clear everything
					data.setData(new Vector<String>(), new Vector<TableDataInterpreter.Entry>());
					assertsList.setListData(new String[] {});
					assertsList.setEnabled(true);
					vars.setEnabled(true);
					exps.setEnabled(true);
					atomic.setEnabled(true);
					load.setEnabled(true);
					reload.setEnabled(true);
					resultsFile = null;
					
					//set title
					frame.setTitle("Error Loading " + fileName);
					
					//remove busy cursor
					frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
					
					//remove loading screen
    				//}
    			}
		
		};
		
		worker.execute();
		
	}
	
	public static void main(String[] args)
	{
	
		InterpreterGUI gui = new InterpreterGUI();
		
		if (args.length == 0) 
		{
		
			//System.out.println("Usage: java InterpreterGUI source -r=FDRoutputForReading -w=FDROutputToWrite -c=CSPFileToCreate");
			gui.run();
		}
		else
		{
			String source = args[0];
			String readFDRFile = null;
			String writeFDRFile = null;
	
			String CSPFile = CSPFILE;
			
			for (int i = 1; i < args.length; i++)
			{
				if (args[i].startsWith("-r="))
					readFDRFile = args[i].substring(3);
				else if (args[i].startsWith("-w="))
					writeFDRFile = args[i].substring(3);
				else if (args[i].startsWith("-c="))
					CSPFile = args[i].substring(3);
				else
					{ System.out.println("argument " + args[i] + " not recognised");return;}
			}
			
			
			gui.run(source,CSPFile,readFDRFile,writeFDRFile);
		}
	}
	
	static void trace(String s){if (DEBUG) System.out.println("InterpreterGUI:" + s);}
}
