import java.util.Map;
import java.io.PrintStream;


public interface InterpreterOutput
{

	void startAssertion(Assertion assertion, boolean result);
	void endTrace();

	void bVarEval(String name, boolean value, int proc);
	void iVarEval(String name, int value, int proc);

	void bVarWrite(String name, boolean value, int proc);
	void iVarWrite(String name, int value, int proc);
	
	void iExprStart(int exp);
	void iExprEnd(int exp, int result);
	void bExprStart(int exp);
	void bExprEnd(int exp, boolean result);
	
	
	void startAtom(int proc);
	void endAtom(int proc);
	
	void signal(String signal, int proc);
	void iSignal(String signal, int value, int proc);
	
	//void outofrange();
	//void assertionfailed();
	
	void uninterpreted(String line);
	
	//this saves creating lots of empty mappings
	static final Map<String, Boolean> emptyBMap = new java.util.TreeMap<String,Boolean>();

}

class BasicInterpreter implements InterpreterOutput
{

	PrintStream stream;
	Assertion current;
	Map<String, Integer> consts;

	public void startAssertion(Assertion assertion, boolean result)
	{
	
		stream.println("Checking " + assertion);
		if (result)
		{
			stream.println("\tresult: true");
		}
		else
		{
			stream.println("\tresult: false\n\tCounterExample:\n");
			current = assertion;
		}
	}
	
	public void endTrace(){current = null;}
		
	public void bVarEval(String name, boolean value, int proc)
	{ stream.println("\t" + name + " is " + value + " in " + current.process(proc)); }
	public void iVarEval(String name, int value, int proc)
	{ stream.println("\t" + name + " is " + value + " in " + current.process(proc)); }
	
	public void bVarWrite(String name, boolean value, int proc)
	{ stream.println("\t" + name + " assigned " + value + " by " + current.process(proc)); }
	
	public void iVarWrite(String name, int value, int proc)	
	{ stream.println("\t" + name + " assigned " + value + " by " + current.process(proc)); }
	
	
	public void iExprStart(int exp)
	{
		stream.println("\t" + current.ownerOfIExpr(exp, consts) + " requests value of " + current.getIExpr(exp, consts));
	}
	public void bExprStart(int exp)
	{
		stream.println("\t" + current.ownerOfBExpr(exp, consts) + " requests value of " + current.getBExpr(exp, consts, emptyBMap));
	}
	
	public void iExprEnd(int exp, int result)
	{
		stream.println("\t" + current.getIExpr(exp, consts) + " is " + result + " in " + current.ownerOfIExpr(exp,consts));
	}
	
	public void bExprEnd(int exp, boolean result)
	{
		stream.println("\t" + current.getBExpr(exp, consts, emptyBMap) + " is " + result + " in " + current.ownerOfBExpr(exp,consts));
	}
	
	
	public void startAtom(int proc)
	{ stream.println("\t" + current.process(proc) + " enters atomic section"); }
	
	public void endAtom(int proc)
	{ stream.println("\t" + current.process(proc) + " leaves atomic section"); }
	
	
	public void signal(String signal, int proc)
	{ stream.println("\t" + current.process(proc) + " signals " + signal); }	
	
	public void iSignal(String signal, int value, int proc)
	{ stream.println("\t" + current.process(proc) + " signals " + signal + "." + value); }
	
	public void outofrange()
	{stream.println("\tOut Of Range");}
	public void assertionfailed()
	{stream.println("\tAssertion Fails");}

	public void uninterpreted(String line)
	{stream.println("Uninterpreted: " + line);}

	public BasicInterpreter(PrintStream stream, Map<String, Integer> consts)
	{
		if (stream == null) throw new InterpreterException("BasicInterpreter given null stream");
		if (consts == null) throw new InterpreterException("BasicInterpreter given null consts");
		this.stream = stream;
		this.consts = consts;
	}


	
	public static void main(String args[]) throws java.io.FileNotFoundException,InterruptedException, java.io.IOException
	{ 
	
		if (args.length == 0) 
		{
		
			System.out.println("Usage: java BasicInterpreter source -r=FDRoutputForReading -w=FDROutputToWrite -c=CSPFileToCreate -o=OutFile");
			return;
		}
		
		String source = args[0];
		PrintStream stream = System.out;
		
		String readFDRFile = null;
		String writeFDRFile = null;

		String CSPFile = "output.csp";
		
		for (int i = 1; i < args.length; i++)
		{
			if (args[i].startsWith("-r="))
				readFDRFile = args[i].substring(3);
			else if (args[i].startsWith("-w="))
				writeFDRFile = args[i].substring(3);
			else if (args[i].startsWith("-c="))
				CSPFile = args[i].substring(3);
			else if (args[i].startsWith("-o="))
				stream = new PrintStream(args[i].substring(3));
		}
		
		Interpreter interpreter = new Interpreter(source, CSPFile, readFDRFile, writeFDRFile);		
		
		InterpreterOutput output = new BasicInterpreter(stream, interpreter.parser.context.globalConsts);
		
		interpreter.interpret(output);		

	}



}