import java.util.Vector;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;


public class Program extends UnnamedProgram
{

	String name;

	public Program(String name, Vector<Vector<CompilableProc>> procs)
	{
		super(procs);
		
		//removed this line since MonitoredProgram uses a null name to signal that it should generate
		//code for an unnamed program eg. for use in an assertion
		//if (name == null) throw new ParserException("name argument to Program null");
		this.name = name;
	}
	
	public String genCode(Context con)
	{
		return name + " = " + super.genCode(con);
	}
	
	/*public UnnamedProgram copy()
	{
		Vector<Vector<CompilableProc>> newProcs = new Vector<Vector<CompilableProc>>();
		Iterator<Vector<CompilableProc>> i = procs.iterator();
		while (i.hasNext())
			newProcs.add(new Vector<CompilableProc>(i.next()));
	
		
	
		return new Program(newProcs, new String(name));
	}*/
		
}

class UnnamedProgram  implements ProcessContainer
{
	final static boolean DEBUG = false;

	Vector<Vector<CompilableProc>> procs;
	
	public UnnamedProgram(Vector<Vector<CompilableProc>> procs)
	{
		if (procs == null) throw new ParserException("procs argument to UnNamedProgram null");
		this.procs = procs;
	}
	
		
	
	public String genCode(Context con)
	{
		String s = "Compile((<";

		Iterator<Vector<CompilableProc>> j = procs.iterator();
		
		while(j.hasNext())
		{	
			Iterator<CompilableProc> i = j.next().iterator();
			s += "<";
			while (i.hasNext())
			{
				s = s + i.next().genCode(con);
				if (i.hasNext()) s = s + ",";
			}
			if (j.hasNext()) 
				s += ">,"; 
			else
				s += ">";
		}
		s = s + ">, " + con.genInit() + "))";
		return s;
	}
	
	protected class forwardIterator implements Iterator<CompilableProc> 
	{
		Iterator<Vector<CompilableProc>> i;
		Iterator<CompilableProc>j;//j == null => !hasNext()
		
		public forwardIterator()
		{
			i = procs.iterator();
			if (i.hasNext()) j = i.next().iterator();
			else j = null;
		}
		
		public boolean hasNext() //after hasNext() the next element should be proc.elementAt(i).elementAt(j)
		{
			if (j == null) return false;
			else if (j.hasNext()) return true;
			else
			{
				if (i.hasNext()) j = i.next().iterator();
				else j = null;
				return hasNext();
			}
		}
		
		public CompilableProc next()
		{
			if (hasNext())
				return j.next();
			else throw new java.util.NoSuchElementException("forwardIterator: no such element - i = " + i + " j = " + j);
		}
	 	public void remove(){throw new java.lang.UnsupportedOperationException ("forwardIterator - remove unsupported");}	
			
	
		/*int i,j;
		Vector<CompilableProc> vec;//if vec != null then vec = proc.elementAt(i)
		
		public forwardIterator()
		{
			i = 0;
			j = 0;
			if (procs.size() > i) vec = procs.elementAt(i);
			else vec = null;
		}
		
		public boolean hasNext() //after hasNext() the next element should be vec.elementAt(j)
		{
			if (vec != null && vec.size() > j)
				return true;
			else if (i + 1 < procs.size())
			{i++; j = 0; return hasNext();}
			else
				return false;
		}
		
		public CompilableProc next()
		{
			if (hasNext())
				return vec.elementAt(j++);
			else throw new java.util.NoSuchElementException("forwardIterator: no such element - i = " + i + " j = " + j);
		}
	 	public void remove(){throw new java.lang.UnsupportedOperationException ("forwardIterator - remove unsupported");}	
	*/}
	
	protected class reverseIterator implements Iterator<CompilableProc> 
	{
	
		// i < 0 => j < 0
		//and so j >= 0 => i >= 0
		int i,j;
		
		Vector<CompilableProc> vec; // i >= 0 => vec = procs.elemenAt(i);
		
		public reverseIterator()
		{
			i = procs.size() - 1;
			if (i >= 0)
			{
				vec = procs.elementAt(i);
				j = vec.size() - 1;
			}
			else
				j = - 1;
		}
		
		public boolean hasNext() //after hasNext() the next element should be proc.elementAt(i).elementAt(j)
		{
			if (j >= 0)
				return true;
			else if (i <= 0)
				return false;
			else
			{
				i--; vec = procs.elementAt(i); j = vec.size() - 1; return hasNext();
			}
		}
		
		public CompilableProc next()
		{
			if (hasNext())
				return vec.elementAt(j--);
			else throw new java.util.NoSuchElementException("reverseIterator: no such element - i = " + i + " j = " + j);
		}
	 	public void 	remove(){throw new java.lang.UnsupportedOperationException ("forwardIterator - remove unsupported");}	

	/*
		int i,j;
		Vector<CompilableProc> vec;//if vec != null then vec = proc.elementAt(i)
		
		public reverseIterator()
		{
			i = procs.size() - 1;
			if (i >= 0) {vec = procs.elementAt(i);j = vec.size() - 1;}			
			else {vec = null;j = -1;}
		}
		
		public boolean hasNext() //after hasNext() the next element should be vec.elementAt(j)
		{
			if (j >= 0)
				return true;
			else if (i >= 0)
			{vec = procs.elementAt(--i);j = vec.size() - 1;return hasNext();}
			else return false;
		}
		
		public CompilableProc next()
		{
			if (hasNext())
				return vec.elementAt(j--);
			else throw new java.util.NoSuchElementException("reverseIterator: no such element - i = " + i + " j = " + j);
		}
	 	public void 	remove(){throw new java.lang.UnsupportedOperationException ("forwardIterator - remove unsupported");}	
	*/}
	
	public int numIEs(Map<String,Integer> consts)
	{
		int num=0;
		forwardIterator i = new forwardIterator();
		while (i.hasNext())
			num += i.next().numIEs(consts);
		return num;
	}

	public int numBEs(Map<String,Integer> consts)
	{
		int num=0;
		forwardIterator i = new forwardIterator();
		while (i.hasNext())
			num += i.next().numBEs(consts);
		return num;
	}
	
	public int numProcs()
	{
		int num=0;
		Iterator<Vector<CompilableProc>> j = procs.iterator();
		while(j.hasNext())
		{
			num += j.next().size();
		}
		return num;
	}

	public String getIExpr(int i, Map<String, Integer> ivals)
	{
		reverseIterator j = new reverseIterator();
		while (j.hasNext())
		{
			CompilableProc p = j.next();
			if (i <= p.numIEs(ivals))
				return p.getIExpr(i, ivals);
			else
				i = i - p.numIEs(ivals);
		}
		throw new InterpreterException("Program.getIExpr(" + i + ") Not enough IExprs found");
	}
	
	public String getBExpr(int i, Map<String, Integer> ivals,Map<String,Boolean> bvals)
	{
		reverseIterator j = new reverseIterator();
		while (j.hasNext())
		{
			CompilableProc p = j.next();
			if (i <= p.numBEs(ivals))
				return p.getBExpr(i,ivals,bvals);
			else
				i = i - p.numBEs(ivals);
		}
		throw new InterpreterException("Program.getIExpr(" + i + ") Not enough BExprs found");
	}
	
	public String process(int i) 
	{
	
		Iterator<CompilableProc> j = new reverseIterator();
		for (int k = 0; k < i; k++)
			j.next();
		return j.next().toString();
	}
	
	
	public String ownerOfIExpr(int i, Map<String,Integer> consts) 
	{
		reverseIterator j = new reverseIterator();
		while (j.hasNext())
		{
			CompilableProc p = j.next();
			if (i <= p.numIEs(consts))
				return p.toString();
			else
				i = i - p.numIEs(consts);
		}
		throw new InterpreterException("Not enough IExprs found"); 
	 
	}
	
	public String ownerOfBExpr(int i, Map<String,Integer> consts)	
	{
		reverseIterator j = new reverseIterator();
		while (j.hasNext())
		{
			CompilableProc p = j.next();
			if (i <= p.numBEs(consts))
				return p.toString();
			else
				i = i - p.numBEs(consts);
		}
		throw new InterpreterException("Not enough BExprs found");
	}

	public int indexOfBExprOwner(int i,Map<String,Integer> consts)
	{
		int k = 0;
		reverseIterator j = new reverseIterator();
		while (j.hasNext())
		{
			CompilableProc p = j.next();
			if (i <= p.numBEs(consts))
				{trace("indexOfBExprOwner:returning index " + k + " for process " + p.toString()); return k;}
			else
			{	i = i - p.numBEs(consts);k++;}
		}
		throw new InterpreterException("Not enough BExprs found");
	}
	public int indexOfIExprOwner(int i,Map<String,Integer> consts)
	{
		int k = 0;
		reverseIterator j = new reverseIterator();
		while (j.hasNext())
		{
			CompilableProc p = j.next();
			if (i <= p.numIEs(consts))
				{trace("indexOfIExprOwner:returning index " + k + " for process " + p.toString()); return k;}
			
			else
			{	i = i - p.numIEs(consts);k++;}
		}
		throw new InterpreterException("Not enough IExprs found"); 
	 
	}
	
	static void trace(String s)
	{
		if (DEBUG) System.out.println("UnnamedProgram: " + s);
	}

	public UnnamedProgram copy()
	{
		Vector<Vector<CompilableProc>> newProcs = new Vector<Vector<CompilableProc>>();
		Iterator<Vector<CompilableProc>> i = procs.iterator();
		while (i.hasNext())
			newProcs.add(new Vector<CompilableProc>(i.next()));
	
		return new UnnamedProgram(newProcs);
	}

}

class MonitoredProgram extends Program
{

	final static boolean DEBUG = true;

	protected CompilableProc monitor;
	protected Vector<VName> vars;
	
	public MonitoredProgram(String name, Vector<Vector<CompilableProc>> procs, CompilableProc monitor, Vector<VName> vars)
	{
		super(name, procs);
		this.monitor= monitor;
		this.vars=vars;
	}
	
	public String genCode(Context con)
	{	
		String s = name != null ? name + " = " : "";
	
		s += "MonitorCompile((<";

		Iterator<Vector<CompilableProc>> j = procs.iterator();
		
		while(j.hasNext())
		{	
			Iterator<CompilableProc> i = j.next().iterator();
			s += "<";
			while (i.hasNext())
			{
				s = s + i.next().genCode(con);
				if (i.hasNext()) s = s + ",";
			}
			if (j.hasNext()) 
				s += ">,"; 
			else
				s += ">";
		}
		s = s + ">, (" + monitor.genCode(con) + ", {";	
		Iterator<VName> i = vars.iterator();
		while (i.hasNext())
		{	
			s += i.next().genCode(con);
			if (i.hasNext())
				s+=",";
		}
		 s += "})," + con.genInit() + "))";
		

		return s;
	}
	
	
	public UnnamedProgram copy()
	{
		Vector<Vector<CompilableProc>> newProcs = new Vector<Vector<CompilableProc>>();
		Iterator<Vector<CompilableProc>> i = procs.iterator();
		while (i.hasNext())
			newProcs.add(new Vector<CompilableProc>(i.next()));
	
		
		//should be safe to return copy without creating a new copy of monitor/vars
		//since we are returning it as an unnamedprogram which does not have monitor or vars
		return new MonitoredProgram(null, newProcs, monitor, vars);
	}
	
	
	public int numIEs(Map<String,Integer> consts)
	{
		return super.numIEs(consts) + monitor.numIEs(consts);
	}

	public int numBEs(Map<String,Integer> consts)
	{
		return super.numBEs(consts) + monitor.numBEs(consts);	
	}
	
	public int numProcs()
	{
	
		return super.numProcs() + 1;
	
	}

	public String getIExpr(int i, Map<String, Integer> ivals)
	{
	
		if (i <= super.numIEs(ivals)) return super.getIExpr(i,ivals);
		else return monitor.getIExpr(i - super.numIEs(ivals), ivals);
	}
	
	public String getBExpr(int i, Map<String, Integer> ivals,Map<String,Boolean> bvals)
	{
		if (i <= super.numBEs(ivals)) return super.getBExpr(i,ivals,bvals);
		else return monitor.getBExpr(i - super.numBEs(ivals), ivals,bvals);
	}
	
	public String process(int i) 
	{
		if (i == super.numProcs()) return monitor.toString();
		else return super.process(i);
	}
	
	
	public String ownerOfIExpr(int i, Map<String,Integer> consts) 
	{
		if (i <= super.numIEs(consts)) return super.ownerOfIExpr(i,consts);
		else return monitor.toString();
	
	}
	
	public String ownerOfBExpr(int i, Map<String,Integer> consts)	
	{
		if (i <= super.numBEs(consts)) return super.ownerOfBExpr(i,consts);
		else return monitor.toString();
	}

	public int indexOfBExprOwner(int i,Map<String,Integer> consts)
	{
		if (i <= super.numBEs(consts)) return super.indexOfBExprOwner(i,consts);
		else return super.numProcs();
	}
	
	public int indexOfIExprOwner(int i,Map<String,Integer> consts)
	{
		if (i <= super.numIEs(consts)) return super.indexOfIExprOwner(i,consts);
		else return super.numProcs();
	}

	static void trace (String s) {if (DEBUG) System.out.println("MonitoredProgram: " + s);}

}



class InitializedProc implements CompilableProc
{
	boolean DEBUG = true;
	
	void trace(String s){if (DEBUG) System.out.println("InitializedProc: " + s);}

	Process proc;
	Vector<Integer> args;
	
	Map<String, Integer> argMap;
	
	public InitializedProc(Process proc, Vector<Integer>args)
	{
		if (proc == null) throw new ParserException("proc argument to InitializedProc null");
		if (args == null) throw new ParserException("args argument to InitializedProc null");
		if (args.size() != proc.consts.size()) throw new ParserException("InitializedProc: args.size() and proc.consts.size() do not match");
		trace("New Initilized Proc - proc: " + proc + " args: " + args);

		this.proc = proc; this.args=args;
	
		argMap = new TreeMap<String, Integer>();
		
		int i = 0;
		while (i < args.size())
		{
			argMap.put(proc.consts.elementAt(i), args.elementAt(i));
			i++;
		}
	}
	
	public String genCode(Context con){return genCode();}
	
	public String genCode()
	{
		String s = proc.name + "(";
			
		Iterator<Integer> j= args.iterator();
		while (j.hasNext())
		{
			Integer next = j.next();
			s = s + next;
			if (j.hasNext()) s = s + ",";
		}
		s = s + ")";
		return s;
	}

	public int numIEs(Map<String,Integer> consts)
	{
		Map<String, Integer> newConsts = new TreeMap<String,Integer>();
		newConsts.putAll(consts);
		newConsts.putAll(argMap);
		return proc.numIEs(newConsts);
	}
	public int numBEs(Map<String,Integer> consts)
	{
		Map<String, Integer> newConsts = new TreeMap<String,Integer>();
		newConsts.putAll(consts);
		newConsts.putAll(argMap);
		return proc.numBEs(newConsts);
	}

	//we add this process's arguments to the valuation passed in
	public String getIExpr(int i, Map<String, Integer> ivals)
	{
		Map<String, Integer> newIvals = new TreeMap<String,Integer>();
		newIvals.putAll(ivals);
		newIvals.putAll(argMap);
		return proc.getIExpr(i,newIvals);
	}
	public String getBExpr(int i, Map<String, Integer> ivals,Map<String,Boolean> bvals) 
	{
		Map<String, Integer> newIvals = new TreeMap<String,Integer>();
		newIvals.putAll(ivals);
		newIvals.putAll(argMap);
		
		return proc.getBExpr(i,newIvals,bvals);	
	}
	
	public String toString()
	{ return this.genCode();}
	
	
}









