import java.util.Vector;
import java.util.Iterator;
import javax.swing.table.*;
import javax.swing.event.*;
//import java.awt.event.*;
import java.util.EventListener;

public class TableData extends AbstractTableModel
{
	
	final static boolean DEBUG = false;
	final static boolean REVERSE = true;
		

	protected boolean showVarReads;
	protected boolean showExprStarts;
	protected boolean showAtomics;
	
	protected Vector<String> headers;
	protected Vector<TableDataInterpreter.Entry> data;

	public TableData(Vector<String> headers, Vector<TableDataInterpreter.Entry> data,boolean showVarReads, boolean showExprStarts,boolean showAtomics)
	{
		trace("constructor");
		this.showVarReads = showVarReads;
		this.showExprStarts = showExprStarts;
		this.showAtomics = showAtomics;
	
		this.headers = headers;
		this.data = data;
	}

	public void setData(Vector<String> headers,Vector<TableDataInterpreter.Entry> data)
	{
		if (data == null) throw new InterpreterException("TableData.setData given null data"); 
		trace("setData");
		this.headers=headers;
		this.data = data;
		fireTableStructureChanged();
		fireTableDataChanged();
	}
	
	public void changeVarReads(boolean show)
	{
		trace("changeVarReads(" + show + ")");
		showVarReads = show;
		//fireTableStructureChanged();
		fireTableDataChanged();
	}
	
	public void changeExprStarts(boolean show)
	{
		trace("changeExprStarts(" + show + ")");
		showExprStarts = show;
		//fireTableStructureChanged();
		fireTableDataChanged();
	}
	
	public void changeAtomics(boolean show)
	{
		trace("changeAtomics(" + show + ")");
		showAtomics = show;
		//fireTableStructureChanged();
		fireTableDataChanged();
	}
	
	protected class filterIterator implements Iterator<TableDataInterpreter.Entry>
	{
		Iterator<TableDataInterpreter.Entry> i;
		TableDataInterpreter.Entry current;
		public filterIterator()
		{	if (data == null) throw new InterpreterException ("TableData.filterIterator.filterIterator(): data == null");
			i = data.iterator(); current = null;}
		
		public TableDataInterpreter.Entry next()
		{
			if (hasNext())
			{
				TableDataInterpreter.Entry e = current;
				current = null;
				return e;
			}
			else
				throw new java.util.NoSuchElementException("TableData.filterIterator.next() called at end of sequence");
		}
		
		public boolean hasNext()
		{
			if (current != null)
				return true;
			else
			{
				while (i.hasNext())
				{
					current = i.next();
					if ( (showVarReads || !current.isVarRead) && (showExprStarts || !current.isExprStart) && (showAtomics || ! current.isAtomic))
						return true;
					else
						current = null;
				}
				return false;
			}
		}
		 
		public void remove() {	throw new java.lang.UnsupportedOperationException("TableData.filterIterator.remove() called");}
	}
	
	public int getRowCount()
	{
		int k = 0;
		filterIterator i = new filterIterator();
		while (i.hasNext())
		{
			k++; i.next();
		}
		trace("getRowCount() returning " + k);
		return k;
	
	}
  	public int getColumnCount()
  	{
  		trace("getColumnCount() returning " + headers.size());
  		return headers.size();
  	}
  	public Object getValueAt(int row, int column)
  	{
  		
  		filterIterator i = new filterIterator();
  		int k = 0;
  		TableDataInterpreter.Entry e = null;
  		while (k <= row)
  		{
  			if (!i.hasNext())
  				{ e = null; break;}
  			e = i.next();
  			k++;
  		}
  		String s = (e != null && e.col == columnIndex(column)) ? e.text :"";
  		if (e == null) trace("getValueAt(" + row + ", " + column + ") e == null so returning " + s);
  		else trace("getValueAt(" + row + ", " + column + ") e.col == " + column + " returning " + s);
  		return s;
  	}
  	
  	public String getColumnName(int column)
  	{	
  		String s =  headers.elementAt(columnIndex(column));
  		trace("getColumnName(" + column + ") returning " + s);
  		return s;
  	}
  	
  	protected int columnIndex(int column)
  	{
  		return REVERSE ? getColumnCount() -1 - column : column;
  	}
  	
  //	public Class<?> getColumnClass(int columnIndex) {trace("getColumnClass"); return getValueAt(0, columnIndex).getClass(); /*super.getColumnClass(columnIndex);*/}
  /*	public void addTableModelListener(TableModelListener l) { trace("addTableModelListener(l)"); super.addTableModelListener(l);}
        public int findColumn(String columnName){ trace("addfindColumn"); return super.findColumn(columnName);}
 	public void fireTableCellUpdated(int row, int column) {trace("fireTableCellUpdated"); super.fireTableCellUpdated(row,column);}
       	public void fireTableChanged(TableModelEvent e) {trace("fireTableChanged"); super.fireTableChanged(e);}
       	public void fireTableDataChanged() {trace("fireTableDataChanged"); super.fireTableDataChanged();}
       	public void fireTableRowsDeleted(int firstRow, int lastRow) {trace("fireTableRowsDeleted"); super.fireTableRowsDeleted(firstRow,lastRow);}
        public void fireTableRowsInserted(int firstRow, int lastRow){trace("fireTableRowsInserted"); super.fireTableRowsInserted(firstRow,lastRow);}
        public void fireTableRowsUpdated(int firstRow, int lastRow) {trace("fireTableRowsUpdated"); super.fireTableRowsUpdated(firstRow,lastRow);}
        public void fireTableStructureChanged(){trace("fireTableStructureChanged"); super.fireTableStructureChanged();}
        public <T extends EventListener> T[] getListeners(Class<T> listenerType) {trace("getListeners"); return super.getListeners(listenerType);}
 	public TableModelListener[] getTableModelListeners() {trace("getTableModelListeners"); return super.getTableModelListeners();}
        public boolean 	isCellEditable(int rowIndex, int columnIndex) {trace("isCellEditable"); return super.isCellEditable(rowIndex,columnIndex);}
        public void removeTableModelListener(TableModelListener l) {trace("removeTableModelListener"); super.removeTableModelListener(l);}
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {trace("setValueAt"); super.setValueAt(aValue,rowIndex,columnIndex);} 
  */	
  	
  	
  	
  	static void trace(String s) {if (DEBUG) System.out.println("TableData: " + s);}
	
}