%%

%byaccj

%{
  private Parser yyparser;
  
  final boolean DEBUG = true;
  Context context;
  int lineno = 0;
  public Yylex(java.io.Reader r, Parser yyparser, Context context) {
    this(r);
    this.yyparser = yyparser;
    this.context = context;
  }

  private void trace(String s)
  {
	if (DEBUG)
		System.out.println("Lexer: " + s);
  }
%}

NUM = [0-9]+
LETTER = [A-Za-z]
NAME = {LETTER}({LETTER}|[0-9])*
NL  = \r|\n|\r\n|\u2028|\u2029|\u000B|\u000C|\u0085
COMMENT = "//".*
CSP = "%%".*

%%

/* operators */
"+" | "-" | "*" | "/" | "%" | "<" | ">" |"[" | "]" | 
"{" | "}" | "(" | ")" | "!" | "," | "=" | ";"| "." 
   { trace(yytext());return (int) yycharat(0); }

">=" {trace("GE");return Parser.GE; }
"<=" {trace("LE");return Parser.LE; }
"&&" {trace("AND");return Parser.AND; }
"||" {trace("OR");return Parser.OR; }
"true" {trace("TRUE");return Parser.TRUE;}
"false" {trace("FALSE");return Parser.FALSE;}

"skip" {trace("SKIP");return Parser.SKIP; }
"iter" {trace("ITER");return Parser.ITER; }
"while" {trace("WHILE");return Parser.WHILE; }
"do" {trace("DO");return Parser.DO; }
"if" {trace("IF");return Parser.IF; }
"then" {trace("THEN");return Parser.THEN; }
"else" {trace("ELSE");return Parser.ELSE; }
":=" {trace("ASSIGN");return Parser.ASSIGN; }
"sig" {trace("SIG");return Parser.SIG; }
"isig" {trace("ISIG");return Parser.ISIG; }
"atomic" {trace("ATOMIC");return Parser.ATOMIC; }
"int" {trace("INT");return Parser.INT; }
"bool" {trace("BOOL"); return Parser.BOOL; }
"assert" {trace("ASSERT"); return Parser.ASSERT;}
"in" {trace("IN"); return Parser.IN;}
"always" {trace("ALWAYS"); return Parser.ALWAYS;}
"never" {trace("NEVER"); return Parser.NEVER;}
"outofrange" {trace("OUTOFRANGE"); return Parser.OUTOFRANGE;}
"nosignal" {trace("NOSIGNAL"); return Parser.NOSIGNAL;}
"max"	{trace("MAX"); return Parser.MAX;}
"min"	{trace("MIN"); return Parser.MIN;}
"const" {trace("GLOBCONST"); return Parser.GLOBCONST;}
"idle"  {trace("NOOP"); return Parser.NOOP;}
"with"	{trace("WITH"); return Parser.WITH;}
"watching" {trace("WATCHING"); return Parser.WATCHING;}

{ NAME }
{ 
	String name = yytext();
	yyparser.yylval = new ParserVal(name);
	if (context.isI(name))
		{trace("IVAR(" + name+")");return Parser.IVAR;}
	else if (context.isB(name))
		{trace("BVAR(" + name+")");return Parser.BVAR;}
	else if (context.isIA(name))
		{trace("IAVAR(" + name+")");return Parser.IAVAR;}
	else if (context.isBA(name))
		{trace("BAVAR(" + name+")");return Parser.BAVAR;}
	else if (context.isConstant(name))
		{trace("CONST(" + name +")"); return Parser.CONST;}
	else if (context.isSig(name))
		{trace("SIGNAME(" + name +")"); return Parser.SIGNAME;}
	else if (context.isISig(name))
		{trace("ISIGNAME(" + name +")"); return Parser.ISIGNAME;}
	else if (context.isProcess(name))
		{trace("PROCESS(" + name +")"); return Parser.PROCESS;}
	else if (context.isProgram(name))
		{trace("PROGRAM(" + name +")"); return Parser.PROGRAM;}
	else {trace("NAME(" + name+")");return Parser.NAME;}

}

{NUM}  { yyparser.yylval = new ParserVal(Integer.parseInt(yytext()));
	 trace("NUM(" + yytext() + ")");
         return Parser.NUM; }

/* whitespace and comments*/
([\t]|" "|{COMMENT})+ {}

{NL} {lineno++;}

{CSP} { yyparser.yylval = new ParserVal(yytext().substring(2)); trace("CSP: "+ yytext()); return Parser.CSP; }

/* error fallback */
[^]    { System.err.println("Error: unexpected character '"+yytext()+"' near line " + lineno); return -1; }
