%{
  import java.io.*;
  import java.util.Vector;
  import java.util.Collection;
  import java.util.Iterator;
  import java.util.TreeMap;
  import java.util.Map;


%}
      
%token CSP
%token NUM  /* a number */
%token NAME /* a name for a process or signal or variable/constant decleration*/
%token TRUE FALSE
%left ';'
%token SKIP ITER WHILE DO IF THEN ASSIGN SIG ISIG ATOMIC MIN MAX NOOP GLOBCONST
%token PROCESS PROGRAM SIGNAME ISIGNAME IVAR BVAR IAVAR BAVAR CONST INT BOOL WITH WATCHING
%token ASSERT IN ALWAYS NEVER OUTOFRANGE NOSIGNAL
%nonassoc IFX
%nonassoc ELSE

%left OR
%left AND
%nonassoc '<' '>' GE LE '='
%left '!'

%left '-' '+'
%left '*' '/' '%'
%left NEG        /* negation--unary minus */
%%

Input:   /* empty string */ 	{trace("Input - empty string matched");}
       | Stmt Input		{trace("Input - Stmt Input matched");}
       ;

Stmt: 	  Process 	{trace("Stmt - Process matched\n");}
	| Decleration ';'{trace("Stmt - Decleration matched\n");}
	| Program {trace("Stmt - Program matched\n");}
	| CSPStmt	{trace("Stmt - CSPStmt Matched\n");}
	| Assertion	{trace("Stmt - Assertion matched\n");}
	;

CSPStmt:	CSP	{trace("CSPStmt - CSP(" + $1.sval + ") matched"); csp.add($1.sval);};

Decleration: 	INT INameList			{trace("Decleration - INT INameList matched");}
	     	| BOOL BNameList		{trace("Decleration - BOOL BNameList matched");}
		| INT '[' ']' IANameList	{trace("Decleration - INT[] IANameList matched");}
		| BOOL '[' ']' BANameList	{trace("Decleration - BOOL[] BANameList matched");}
		| SIG SigNameList		{trace("Decleration - SIG[] SigNameList matched");}
		| ISIG ISigNameList		{trace("Decleration - ISIG[] ISigNameList matched");}
		| GLOBCONST GlobConstList 	{trace("Decleration - GLOBCONST GlobConstList matched");}
		;

INameList:	IName					{trace("INameList - IName matched");}
		| IName ',' INameList	{trace("INameList - IName , INameList matched");}
		;

IName:		NAME	{context.newI($1.sval);trace("IName - NAME (" + $1.sval + ") matched");}
		| NAME '=' NUM {context.newI($1.sval, $3.ival);trace("IName - NAME (" + $1.sval + ") = NUM ("+ $3.ival + ") matched");}
		;

IANameList:	IAName			{trace("IANameList - IAName matched");}
		| IAName ',' IANameList	{trace("IANameList - IAName , IANameList matched");}
		;
		
IAName:		NAME	{context.newIA($1.sval);trace("IAName - NAME (" + $1.sval + ") matched");}
		| NAME '=' '<' NonEmptyNumList '>' {context.newIA($1.sval, (Vector<Integer>)$4.obj);trace("IAName - NAME (" + $1.sval + ") = < NonEmptyNumList > matched");}
		;

BNameList:	BName			{trace("BNameList - BName matched");}
		| BName ',' BNameList	{trace("BNameList - BName), BNameList matched");}
		;

BName:		NAME			{context.newB($1.sval);trace("BName - NAME (" + $1.sval + ") matched");}
		|NAME '=' TRUE	{context.newB($1.sval, true);trace("BNameList - NAME (" + $1.sval + ") = TRUE matched");}
		|NAME '=' FALSE	{context.newB($1.sval, false);trace("BNameList - NAME (" + $1.sval + ") = FALSE matched");}
		;

BANameList:	BAName			{trace("BANameList - BAName matched");}
		| BAName ',' BANameList	{context.newBA($1.sval);trace("BANameList - BAName (" + $1.sval + "), BANameList matched");}
		;

GlobConstList:	NAME '=' NUM					{context.newGlobConst($1.sval, $3.ival); trace("GlobConstList - NAME(" + $1.sval + " = " + "NUM(" + $3.ival + ") - matched");}
		|NAME '=' NUM ',' GlobConstList {context.newGlobConst($1.sval, $3.ival); trace("GlobConstList - NAME(" + $1.sval + " = " + "NUM(" + $3.ival + ") , GlobConstList - matched");}
		;
	
BAName:		NAME	{context.newBA($1.sval);trace("BAName - NAME (" + $1.sval + ") matched");}
		|NAME '=' '<' NonEmptyBoolList '>' {context.newBA($1.sval, (Vector<Boolean>)$4.obj); trace("BAName - NAME(" + $1.sval + ") = < NonEmptyBoolList >");}
		;
		
NonEmptyBoolList:	TRUE	{Vector<Boolean> vector = new Vector<Boolean>(); vector.add(true); $$.obj = vector; trace("NonEmptyBoolList - TRUE matched");}
			| NonEmptyBoolList ',' TRUE	{((Vector<Boolean>)$1.obj).add(true); $$.obj = $1.obj; trace("NonEmptyBoolList - NonEmptyBoolList, TRUE matched");}
			| FALSE	{Vector<Boolean> vector = new Vector<Boolean>(); vector.add(false); $$.obj = vector; trace("NonEmptyBoolList - FALSE matched");}
			| NonEmptyBoolList ',' FALSE	{((Vector<Boolean>)$1.obj).add(false); $$.obj = $1.obj; trace("NonEmptyBoolList - NonEmptyBoolList, FALSE matched");}
			;
	

SigNameList:	NAME			{context.newSig($1.sval);trace("SigNameList - NAME (" + $1.sval + ") matched");}
		| NAME ',' SigNameList	{context.newSig($1.sval);trace("SigNameList - NAME (" + $1.sval + "), SigNameList matched");}
		;

ISigNameList:	NAME			{context.newISig($1.sval);trace("ISigNameList - NAME (" + $1.sval + ") matched");}
		| NAME ',' ISigNameList	{context.newISig($1.sval);trace("ISigNameList - NAME (" + $1.sval + "), ISigNameList matched");}
		;

Program: NAME '=' '<' ProcList '>' 	{
						Vector<Vector<CompilableProc>> vec = new Vector<Vector<CompilableProc>>();
						vec.add((Vector<CompilableProc>) $4.obj);
						progs.put($1.sval,new Program( $1.sval,vec));
						context.newProgram($1.sval);
						trace("Program - NAME(" + $1.sval + ") = < ProcList > matched");
				 	}
         | NAME '=' '<' ProcListList '>'{
						progs.put($1.sval,new Program( $1.sval,(Vector<Vector<CompilableProc>>) $4.obj));
						context.newProgram($1.sval);
						trace("Program - NAME(" + $1.sval + ") = < ProcListList > matched");
					}
         |NAME '=' '<' ProcList '>' WITH Proc WATCHING '{' VarNameList '}'	
         				{
						Vector<Vector<CompilableProc>> vec = new Vector<Vector<CompilableProc>>();
						vec.add((Vector<CompilableProc>) $4.obj);
						progs.put($1.sval,new MonitoredProgram( $1.sval,vec, (CompilableProc) $7.obj, (Vector<VName>)$10.obj ));
						context.newProgram($1.sval);
						trace("Program - NAME(" + $1.sval + ") = < ProcList > WITH Proc WATCHING '{' VarNameList '}' matched");
				 	}
         | NAME '=' '<' ProcListList '>' WITH Proc WATCHING '{' VarNameList '}'
         				{
						progs.put($1.sval,new MonitoredProgram( $1.sval,(Vector<Vector<CompilableProc>>) $4.obj, (CompilableProc) $7.obj, (Vector<VName>)$10.obj));
						context.newProgram($1.sval);
						trace("Program - NAME(" + $1.sval + ") = < ProcListList > WITH Proc WATCHING '{' VarNameList '}' matched");
					}
         ;

ProcListList: '<' ProcList '>' { Vector<Vector<CompilableProc>> vec = new Vector<Vector<CompilableProc>>(); vec.add((Vector<CompilableProc>) $2.obj); $$.obj=vec;trace("ProcListList - '<' ProcList '>' matched");}
              | ProcListList ',' '<' ProcList '>' { ((Vector<Vector<CompilableProc>>)$1.obj).add((Vector<CompilableProc>)$4.obj); $$.obj = $1.obj;trace("ProcListList - ProcListList ',' '<' ProcList '>' matched");}


/*ProcList: 	Proc 			{trace("ProcList -Proc $1.obj: " + $1.obj.toString());
					 Vector<InitializedProc> vec = new Vector<InitializedProc>();
					 vec.add( (InitializedProc) $1.obj);
					 $$.obj = vec;
					 trace("ProcList - Proc matched");
					 }
		| ProcList ',' Proc 	{//trace("ProcList -Proc,ProcList $1.obj: " + $1.obj.toString());
					//trace("ProcList -Proc,ProcList $3.obj: " + $3.obj.toString());
					if (! ($1.obj instanceof Vector)) throw new ParserException("ProcList - ProcList,Proc $1.obj is not a Vector");
					if (!( $3.obj instanceof InitializedProc)) throw new ParserException("ProcList - ProcList,Proc $3.obj is not an InitializedProc");
					((Vector<InitializedProc>)$1.obj).add((InitializedProc) $3.obj); 
					$$.obj = $1.obj; trace("ProcList - ProcList, Proc matched");}
		;

Proc:	PROCESS '(' NumList ')'	
		{
			if (procs.get($1.sval).numArgs() != ((Vector<Integer>)$3.obj).size()) throw new ParserException($1.sval + "expects " + procs.get($1.sval).numArgs() + " but has been given " + ((Vector<Integer>)$3.obj).size());
			//$$.sval = $1.sval;
			//$$.obj = $3.obj;
			$$.obj = new InitializedProc(procs.get($1.sval), (Vector<Integer>)$3.obj);
			trace("Proc $$.obj: " + $$.obj.toString());
			trace("Proc - PROCESS(" + $1.sval + ") ( NumList ) matched");
		};

*/

ProcList: 	Proc
		{
			check( $1.obj instanceof InitializedProc, "ProcList - Proc $1.obj incorrect type");
			
			Vector<CompilableProc> vec = new Vector<CompilableProc>();
			InitializedProc p = (InitializedProc) $1.obj;
			
			vec.add(p);
			$$.obj = vec;
			trace("ProcList - Proc matched");
		}
		| ProcList ',' Proc
		{
			check( $3.obj instanceof InitializedProc, "ProcList - Proc,ProcList $1.obj incorrect type");
			check( $1.obj instanceof Vector, "ProcList - Proc,ProcList $3.obj incorrect type");
			
			Vector<CompilableProc> vec = (Vector<CompilableProc>)$1.obj;
			InitializedProc p = (InitializedProc) $3.obj;
			
			vec.add(p);
			$$.obj = vec;
			trace("ProcList - ProcList , Proc matched");
		}
		;


Proc: 	PROCESS '(' NonEmptyNumList ')'
	{
		check( $3.obj instanceof Vector, "Proc - PROCESS(NonEmptyNumList) $3.obj incorrect type");
		
		Process process = procs.get($1.sval);
		Vector<Integer> args = (Vector<Integer>)$3.obj;
		//trace("process = " + process);
		//trace("args = " + args);
		if (process.numArgs() != args.size()) throw new ParserException($1.sval + " given " + args.size() + " arguments but expects " + process.numArgs());
		
		$$.obj = new InitializedProc(process, args);
		trace("Proc - PROCESS ( NonEmptyNumList ) matched");
		//trace("Proc - returning " + $$.obj);
	}		
	|PROCESS '('')'
	{
		Process process = procs.get($1.sval);
		//trace("process = " + process);
		
		if (process.numArgs() != 0) throw new ParserException($1.sval + " given no arguments but expects " + process.numArgs());
		
		$$.obj = new InitializedProc(process, new Vector<Integer>());
		trace("Proc - PROCESS () matched");
		//trace("Proc - returning " + $$.obj);
	}
	;

NonEmptyNumList:	NUM	
			{
				Vector<Integer> vector = new Vector<Integer>();
				vector.add($1.ival);
				$$.obj = vector;
				trace("NonEmptyNumList - NUM(" + $1.ival + ") matched");
				//trace("NonEmptyNumList returning " + $$.obj);
			}
			| NonEmptyNumList ',' NUM	
			{	
				check( $1.obj instanceof Vector, "NonEmptyNumList - NonEmptyNumList, Num $1.obj incorrect type");
				((Vector<Integer>)$1.obj).add($3.ival); 
				$$.obj = $1.obj; 
				trace("NonEmptyNumList - NonEmptyNumList, NUM(" + $3.ival + ") mathced");
				//trace("NonEmptyNumList returning " + $$.obj);
			}
			;

Process: NAME '(' ArgList ')' {context.registerConstants((Vector<String>)$3.obj); trace("Registering Constants for " + $1.sval);} '=' Prog	{ 	procs.put($1.sval, new Process($1.sval, (Vector<String>)$3.obj, (Cmd)$7.obj));
				context.newProcess($1.sval);
				trace("Process - NAME(" + $1.sval + ") (ArgList) = Prog matched");
				context.deregisterConsts();//needs doing before next arglist is found
							    //as lexer should flag names in arglist as names not consts	
			}
	 ;

ArgList:				{$$.obj = new Vector<String>(); trace("ArgList - Empty String matched\n");}
		| NonEmptyArgList 	{$$.obj = $1.obj;trace("ArgList - NonEmptyArgList matched\n");}
		;

NonEmptyArgList: NAME				{Vector<String> v = new Vector<String>(); v.add($1.sval); $$.obj = v;trace("NonEmptyArgList - Name(" + $1.sval + ") matched");}
		|NonEmptyArgList ',' NAME 	{ ((Vector<String>)$1.obj).add($3.sval); $$.obj = $1.obj;trace("NonEmptyArgList - Name(" + $1.sval + "), NonEmptyArgList matched");}
		;


VarNameList:	VarName 			{Vector<VName> v = new Vector<VName>(); v.add((VName)$1.obj); $$.obj = v;trace("VarNameList - VarName matched");}
		| VarNameList ',' VarName	{Vector<VName> v = (Vector<VName>)$1.obj; v.add((VName)$3.obj); $$.obj = v;trace("VarNameList - VarNameList , VarName matched");}
		;

VarName:	BVName	{$$.obj = $1.obj; trace("VarName - BVName matched");}
		| IVName  {$$.obj = $1.obj; trace("VarName - IVName matched");}
		;

Prog:	Cmd		{$$.obj = $1.obj;trace("Prog - Cmd matched");}
	| '{' Cmds '}'	{$$.obj = $2.obj;trace("Prog - { Cmds } matched");}
	;

Cmds:	Cmd		{$$.obj = $1.obj; trace("Cmds - Cmd matched");}
	| Cmd ';'	{$$.obj = $1.obj; trace("Cmds - Cmd ; matched");}
	| Cmd ';' Cmds	{$$.obj = new Seq( (Cmd)$1.obj, (Cmd)$3.obj); trace("Cmds - Cmd;Cmd matched\n");}
	;

Cmd:	SKIP				{$$.obj = new Skip(); trace("Cmd - Skip matched\n");}
	| ITER Prog			{$$.obj = new Iter( (Cmd) $2.obj); trace("Cmd - ITER Prog matched\n");}
	| WHILE BExpr DO Prog		{$$.obj = new While( (BExpr) $2.obj, (Cmd) $4.obj); trace("Cmd - WHILE BExpr DO Prog matched\n");}
	| DO Prog WHILE BExpr		{$$.obj = new DoWhile( (Cmd) $2.obj, (BExpr) $4.obj); trace("Cmd - DO Prog WHILE BExpr matched\n");}
	| IF BExpr THEN Prog %prec IFX 	{$$.obj = new If( (BExpr) $2.obj, (Cmd) $4.obj); trace("Cmd - IF BExpr THEN Prog matched\n");} 
	| IF BExpr THEN Prog ELSE Prog	{$$.obj = new If( (BExpr) $2.obj, (Cmd) $4.obj, (Cmd) $6.obj); trace("Cmd - IF BExpr THEN Prog Else Prog matched\n");}
	| IVName ASSIGN IExpr		{$$.obj = new IAssign( (IVName) $1.obj, (IExpr) $3.obj); trace("Cmd - IVName ASSIGN IExpr matched\n");}
	| BVName ASSIGN BExpr		{$$.obj = new BAssign( (BVName) $1.obj, (BExpr) $3.obj); trace("Cmd - BVName ASSIGN BExpr matched\n");}
	| SIG '(' SIGNAME ')'		{$$.obj = new Sig($3.sval); trace("Cmd - Sig(" + $3.sval + ")  matched\n");}
	| ISIG '(' ISIGNAME ',' IExpr ')'{$$.obj = new ISig($3.sval, (IExpr) $5.obj); trace("Cmd - Sig(" + $3.sval + ", IEXpr)  matched\n");}
	| ATOMIC Prog			{$$.obj = new Atomic((Cmd)$2.obj); trace("Cmd - Atomic Prog matched\n");}
	| NOOP				{$$.obj = new NoOp(); trace("Cmd - NOOP matched\n");}
	;

IVName:	IVAR			{ $$.obj = new IVarName($1.sval); trace("IVName - IVAR (" + $1.sval + ") matched");}
	| IAVAR '[' IConst ']'	{ $$.obj = new IArrayName($1.sval, (IConst) $3.obj); trace("IVName - IAVAR (" + $1.sval + ") [IConst] matched");}
	;

BVName:	BVAR			{ $$.obj = new BVarName($1.sval); trace("BVName - BVAR (" + $1.sval + ") matched");}
	| BAVAR '[' IConst ']'	{ $$.obj = new BArrayName($1.sval, (IConst) $3.obj); trace("BVName - BAVAR (" + $1.sval + ") [IConst] matched");}
	;

IConst: CONST			{ $$.obj = new VarIConst($1.sval); trace("IConst - CONST (" + $1.sval + ") matched");}
	| NUM			{ $$.obj = new NumIConst($1.ival); trace("IConst -  NUM (" + $1.ival + ") matched");}
	| IConst '+' IConst	{ $$.obj = new AddConst( (IConst)$1.obj, (IConst) $3.obj); trace("IConst - IConst + IConst matched");}
	| IConst '-' IConst	{ $$.obj = new SubConst( (IConst)$1.obj, (IConst) $3.obj); trace("IConst - IConst - IConst matched");}
	| IConst '*' IConst	{ $$.obj = new MultConst((IConst)$1.obj, (IConst) $3.obj); trace("IConst - IConst * IConst matched");}
	| IConst '/' IConst	{ $$.obj = new DivConst( (IConst)$1.obj, (IConst) $3.obj); trace("IConst - IConst / IConst matched");}
	| IConst '%' IConst	{ $$.obj = new ModConst( (IConst)$1.obj, (IConst) $3.obj); trace("IConst - IConst % IConst matched");}
	| '-' IConst %prec NEG	{ $$.obj = new NegIConst((IConst)$2.obj); trace("IConst - -IConst matched");}
	| '(' IConst ')'	{ $$.obj = $2.obj; trace ("IConst - (IConst) matched");}
	;

BExpr: '!' BExpr		{$$.obj = new Not( (BExpr) $2.obj); trace("BExpr -  !BExpr matched");}
	| BExpr AND BExpr	{$$.obj = new And( (BExpr) $1.obj, (BExpr) $3.obj);trace("BExpr - BExpr AND BExpr matched");}
	| BExpr OR BExpr	{$$.obj = new Or( (BExpr) $1.obj, (BExpr) $3.obj);trace("BExpr - BExpr OR BExpr matched");}
	| IExpr '=' IExpr	{$$.obj = new Eq( (IExpr) $1.obj, (IExpr) $3.obj); trace("BExpr - IExpr = IExpr matched");}
	| IExpr GE IExpr	{$$.obj = new Ge( (IExpr) $1.obj, (IExpr) $3.obj); trace("BExpr - IExpr GE IExpr matched");}
	| IExpr '>' IExpr	{$$.obj = new Gt( (IExpr) $1.obj, (IExpr) $3.obj); trace("BExpr - IExpr > IExpr matched");}
	| IExpr LE IExpr	{$$.obj = new Le( (IExpr) $1.obj, (IExpr) $3.obj);  trace("BExpr - IExpr LE IExpr matched");}
	| IExpr '<' IExpr	{$$.obj = new Lt( (IExpr) $1.obj, (IExpr) $3.obj);  trace("BExpr - IExpr < IExpr matched");}
	| '(' BExpr ')'		{$$.obj = $2.obj;  trace("BExpr - (BExpr) matched");}
	| TRUE			{$$.obj = new True();  trace("BExpr - TRUE matched");}
	| FALSE			{$$.obj = new False(); trace("BExpr - FALSE matched");}
	| BVar			{$$.obj = $1.obj; trace("BExpr - BVar matched");}
	;

BVar:	BVAR			{ $$.obj = new BVar($1.sval);  trace("BVar - BVAR("+$1.sval+") matched");}
	| BAVAR '[' IConst ']'	{ $$.obj = new BArray($1.sval, (IConst) $3.obj);trace("BVar - BVAR("+$1.sval+") [IConst]  matched");}
	;

IExpr:	IExpr '+' IExpr		{$$.obj = new Plus( (IExpr) $1.obj, (IExpr) $3.obj);trace("IExpr - IExpr + IExpr matched");}
	|IExpr '-' IExpr	{$$.obj = new Minus( (IExpr) $1.obj, (IExpr) $3.obj);trace("IExpr - IExpr - IExpr matched");}
	|IExpr '*' IExpr	{$$.obj = new Times( (IExpr) $1.obj, (IExpr) $3.obj);trace("IExpr - IExpr * IExpr matched");}
	|IExpr '/' IExpr	{$$.obj = new Div( (IExpr) $1.obj, (IExpr) $3.obj);trace("IExpr - IExpr / IExpr matched");}
	|IExpr '%' IExpr	{$$.obj = new Mod( (IExpr) $1.obj, (IExpr) $3.obj);trace("IExpr - IExpr % IExpr matched");}
	|MAX '(' IExpr ',' IExpr ')' {$$.obj = new Max( (IExpr)$3.obj, (IExpr)$5.obj); trace("IExpr - MAX ( IExpr, IExpr) matched");}
	|MIN '(' IExpr ',' IExpr ')' {$$.obj = new Min( (IExpr)$3.obj, (IExpr)$5.obj); trace("IExpr - MIN ( IExpr, IExpr) matched");}
	|'-' IExpr %prec NEG	{$$.obj = new Neg( (IExpr) $2.obj);trace("IExpr - -IExpr matched");}
	| '(' IExpr ')'		{$$.obj = $2.obj; trace("IExpr - (IExpr) matched");}
	| CONST			{ $$.obj = new IVarConst($1.sval); trace("IExpr - CONST (" + $1.sval + ") matched");}
	| NUM			{ $$.obj = new NumConst($1.ival); trace("IExpr -  NUM (" + $1.ival + ") matched");}
	| IVar			{$$.obj = $1.obj;trace("IExpr - IVar matched");}
	;

IVar:	IVAR			{ $$.obj = new IVar($1.sval);trace("IVar - IVAR("+$1.sval+") matched");}
	| IAVAR '[' IConst ']'	{ $$.obj = new IArray($1.sval, (IConst) $3.obj);trace("IVar - IVAR("+$1.sval+") [IConst]  matched");}
	;

Assertion:	ASSERT ALWAYS BExpr IN PROGRAM	
		{ 
			asserts.add(new BoolAssertion(progs.get($5.sval),(BExpr)$3.obj));
			trace("Assertion - ASSERT ALWAYS BExpr IN PROGRAM(" + $5.sval + ") matched");
		}
		|ASSERT NEVER BExpr IN PROGRAM	
		{ 
			asserts.add(new BoolAssertion(progs.get($5.sval),new Not((BExpr)$3.obj)));
			trace("Assertion - ASSERT NEVER BExpr IN PROGRAM(" + $5.sval + ") matched");
		}
		|ASSERT NOSIGNAL '{' SignalList '}' IN PROGRAM
		{
			asserts.add(new SignalAssertion(progs.get($7.sval), (Vector<String>)$4.obj));
			trace("Assertion - ASSERT NOSIGNAL { SignalList } IN PROGRAM(" + $7.sval + ")"); 
		}
		;

SignalList: 	Signal			{Vector<String> vec = new Vector<String>(); vec.add($1.sval); $$.obj = vec; trace("SignalList - Signal(" + $1.sval + ") matched");}
		| Signal ',' SignalList	{((Vector<String>)$3.obj).add($1.sval); $$.obj = $3.obj; trace("SignalList - Signal(" + $1.sval + ") , SignalList matched");}
		;
		
Signal:		SIGNAME	{$$.sval = $1.sval; trace("Signal - SIGNAME(" + $1.sval + ") matched");}
		|ISIGNAME '.' NUM {$$.sval = $1.sval + "." + String.valueOf($3.ival); trace("Signal - ISIGNAME(" + $1.sval + ") . NUM(" + String.valueOf($3.ival));}
		| OUTOFRANGE {$$.sval = "outofrange"; trace("Signal - OUTOFRANGE matched");}
		;

%%
  
  Context context; //keeps track of variable names for entire program
  Map<String, Process> procs;
  Vector<String> csp;//needs to be a vector so its in order, so asserts occur in the correct order
  Map<String, Program> progs;
  Vector<Assertion> asserts;

  private Yylex lexer;


  private int yylex (){
    int yyl_return = -1;
    try{
      yylval = new ParserVal(0);
      yyl_return = lexer.yylex();

    }
    catch(IOException e){yyerror(e.getMessage());}
    return yyl_return;
  }


  public void yyerror (String error) {
    throw new ParserException("Error Near Line " + lexer.lineno + ": " + error);
  }


  public Parser(Reader r) {
    context = new Context();
    lexer = new Yylex(r, this, context);
    procs = new TreeMap<String, Process> ();
    csp = new Vector<String>();
    progs = new TreeMap<String, Program>();
    asserts = new Vector<Assertion>();
  }

  public static void main(String args[]) throws IOException
  //reads args[0] outputs to args[1] or std output
  {
	if ( args.length > 0 ) 
	{
		Parser parser;
		// parse a file
		parser = new Parser(new FileReader(args[0]));
		PrintStream stream = (args.length > 1) ? new PrintStream( args[1]) : System.out;
		parser.output(stream);
		stream.close();
	}

   }
   
   public void output(PrintStream stream) throws IOException
   {
   	yyparse();
   	stream.println("include \"share2priorities.csp\"\n");
	stream.println(context.genCode());
	stream.println("nie = " + numIEs());
	stream.println("nbe = " + numBEs());
	stream.println("nps = " + numProcs());
	for (Iterator<String> i = csp.iterator(); i.hasNext(); stream.println(i.next())) {}
	for (Iterator<String> i = procs.keySet().iterator(); i.hasNext(); stream.println(procs.get(i.next()).genCode(context)) ) {}
	for (Iterator<String> i = progs.keySet().iterator(); i.hasNext(); stream.println(progs.get(i.next()).genCode(context)) ) {}
	for (Iterator<Assertion> i = asserts.iterator(); i.hasNext(); stream.println(i.next().genCode(context)) ) {}
	stream.flush();   	
   }
    
   public int numIEs()
   {
   	//search over both assertions and progs in case the user wants to add csp asserts directly
   	//(assuming they perform those asserts on declared programs)
   	int num = 1;//must be at least 1
   	Iterator<Assertion> i = asserts.iterator();
   	while (i.hasNext())
   	{
   		int j = i.next().numIEs(context.globalConsts);
   		num = (j > num)? j : num;
   	}
   	Iterator<String> k = progs.keySet().iterator();
   	while (k.hasNext())
   	{
   		int j = progs.get(k.next()).numIEs(context.globalConsts);
   		num = (j > num)? j : num;
   	}
   	return num;
   }
   
   public int numBEs()
   {
   	int num = 1; //must be at least 1
   	Iterator<Assertion> i = asserts.iterator();
   	while (i.hasNext())
   	{
   		int j = i.next().numBEs(context.globalConsts);
   		num = (j > num)? j : num;
   	}
   	Iterator<String> k = progs.keySet().iterator();
   	while (k.hasNext())
   	{
   		int j = progs.get(k.next()).numBEs(context.globalConsts);
   		num = (j > num)? j : num;
   	}
   	return num;
   }
    
   public int numProcs()
   {
   	int num = 0;
   	Iterator<Assertion> i = asserts.iterator();
   	while (i.hasNext())
   	{
   		int j = i.next().numProcs();
   		num = (j > num)? j : num;
   	}
   	Iterator<String> k = progs.keySet().iterator();
   	while (k.hasNext())
   	{
   		int j = progs.get(k.next()).numProcs();
   		num = (j > num)? j : num;
   	}
   	return num;
   }

   final boolean DEBUG = true;
   private void trace (String s)
   {
	if (DEBUG)
		System.out.println("Parser: " + s);
   }
   private void check(boolean b, String s)
   {
   	if (DEBUG) if (!b) throw new ParserException("assert failed: " + s);
   }
